/*
 *  $Id: netlogreader.cc,v 1.2 2001/11/02 12:38:20 dreibh Exp $
 *
 * SCTP implementation according to RFC 2960.
 * Copyright (C) 1999-2001 by Thomas Dreibholz
 *
 * Realized in co-operation between Siemens AG
 * and University of Essen, Institute of Computer Networking Technology.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * There are two mailinglists available at www.sctp.de which should be used for
 * any discussion related to this implementation.
 *
 * Contact: discussion@sctp.de
 *          dreibh@exp-math.uni-essen.de
 *
 * Purpose: NetLogger File Reader
 *
 */


#include "tdsystem.h"
#include "netlogreader.h"
#include "timedthread.h"
#include "networkmonitorinterface.h"
#include "streammonitorreport.h"
#include "netlogfileheader.h"
#include "tools.h"


namespace Coral {


// ###### Constructor #######################################################
NetLogReader::NetLogReader(const char* name)
{
   // ====== Initialization =================================================
   TimeStamp      = 0;
   Interval       = 0;
   FileDescriptor = fopen(name,"r");
   if(FileDescriptor == NULL) {
      cerr << "ERROR: Unable to read log file <" << name << ">!" << endl;
      return;
   }

   // ====== Read header from file ==========================================
   NetLogFileHeader nlfh;
   if(fread((void *)&nlfh,sizeof(nlfh),1,FileDescriptor) != 1) {
      cerr << "ERROR: Read error!" << endl;
      fclose(FileDescriptor);
      FileDescriptor = NULL;
      return;
   }

   // ====== Check header ===================================================
   if( (strncmp((char*)&nlfh.ID,NetLogFileHeaderID,4)) ||
       (translate16(nlfh.Version) != NetLogFileHeaderVersion) ||
       (translate16(nlfh.EntrySize) != sizeof(NetworkMonitorReport)) ) {
       cerr << "ERROR: File <" << name << "> has invalid format!" << endl;
      fclose(FileDescriptor);
      FileDescriptor = NULL;
      return;       
   }

   TimeStamp         = translate64(nlfh.TimeStamp);
   Interval          = translate64(nlfh.Interval);
   Flags             = translate32(nlfh.Flags);
   StreamReportCount = 0;
}


// ###### Read next report from file ########################################
bool NetLogReader::readNextReport(NetworkMonitorReport* report)
{
   while(StreamReportCount > 0) {
      StreamReport sr;
      readNextStreamReport(&sr);
   }
   if(FileDescriptor != NULL) {
      if(fread((void*)report,sizeof(NetworkMonitorReport),1,FileDescriptor) == 1) {
         report->translate();
         if(Flags & NLFH_ContainsStreams) {
            if(fread((void*)&StreamReportCount,sizeof(card32),1,FileDescriptor) == 1) {
               StreamReportCount = translate32(StreamReportCount);
            }
         }
         return(true);
      }
   }
   return(false);
}


// ###### Read next report from file ########################################
bool NetLogReader::readNextStreamReport(StreamReport* report)
{
   if(FileDescriptor != NULL) {
      if(StreamReportCount > 0) {
         if(fread((void*)report,sizeof(StreamReport),1,FileDescriptor) == 1) {
            report->translate();
            StreamReportCount--;
            return(true);
         }
      }
   }
   return(false);
}


// ###### Destructor ########################################################
NetLogReader::~NetLogReader()
{
   if(FileDescriptor) {
      fclose(FileDescriptor);
      FileDescriptor = NULL;
   }
}


}
