/*
 *  $Id: networkmonitorreport.h,v 1.2 2001/11/02 12:38:20 dreibh Exp $
 *
 * SCTP implementation according to RFC 2960.
 * Copyright (C) 1999-2001 by Thomas Dreibholz
 *
 * Realized in co-operation between Siemens AG
 * and University of Essen, Institute of Computer Networking Technology.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * There are two mailinglists available at www.sctp.de which should be used for
 * any discussion related to this implementation.
 *
 * Contact: discussion@sctp.de
 *          dreibh@exp-math.uni-essen.de
 *
 * Purpose: Network Monitor Report
 *
 */


#ifndef NETWORKMONITORREPORT_H
#define NETWORKMONITORREPORT_H


#include "tdsystem.h"
#include "trafficclassvalues.h"


namespace Coral {


/**
  * This is a network report, which can be read from a NetworkMonitor.
  *
  * @short   Network Monitor Report
  * @author  Thomas Dreibholz (dreibh@exp-math.uni-essen.de)
  * @version 1.0
  */
class NetworkMonitorReport
{
   // ====== Constructor ====================================================
   public:
   /**
     * Constructor.
     */
   NetworkMonitorReport();

   // ====== Byte order translation =========================================
   /**
     * Translate byte order.
     */
   void translate();


   // ====== Status functions ===============================================
   /**
     * Reset report.
     */
   void reset();


   // ======  Report data ===================================================
   /**
     * Interval of the report in microseconds.
     */
   card64 Interval;

   /**
     * Timestamp of this report (microseconds since 01-Jan-1970).
     */
   card64 TimeStamp;

   /**
     * Bytes of raw data transmitted over network interface.
     */
   card64 BytesRaw;

   /**
     * Bytes transmitted in IPv6 packets.
     */
   card64 BytesIPv4;

   /**
     * Bytes transmitted in IPv6 packets.
     */
   card64 BytesIPv6;

   /**
     * Bytes transmitted in TCP packets inclusive TCP header.
     */
   card64 BytesTCP;

   /**
     * Bytes transmitted in TCP packets without TCP headers.
     */
   card64 BytesTCPData;

   /**
     * Bytes transmitted in SCTP packets inclusive SCTP header.
     */
   card64 BytesSCTP;

   /**
     * Bytes transmitted in SCTP packets without SCTP headers.
     */
   card64 BytesSCTPData;

   /**
     * Bytes transmitted in UDP packets inclusive UDP header.
     */
   card64 BytesUDP;

   /**
     * Bytes transmitted in UDP packets without UDP headers.
     */
   card64 BytesUDPData;

   /**
     * Bytes transmitted in ICMP version 4 packets inclusive ICMP header.
     */
   card64 BytesICMP4;

   /**
     * Bytes transmitted in ICMP version 6 packets inclusive ICMP header.
     */
   card64 BytesICMP6;

   /**
     * Number of raw packets transmitted over network interface.
     */
   card32 PacketsRaw;

   /**
     * Number of IPv4 packets transmitted.
     */
   card32 PacketsIPv4;

   /**
     * Number of IPv6 packets transmitted.
     */
   card32 PacketsIPv6;

   /**
     * Number of TCP packets transmitted.
     */
   card32 PacketsTCP;

   /**
     * Number of SCTP packets transmitted.
     */
   card32 PacketsSCTP;

   /**
     * Number of UDP packets transmitted.
     */
   card32 PacketsUDP;

   /**
     * Number of ICMP version 4 packets transmitted.
     */
   card32 PacketsICMP4;

   /**
     * Number of ICMP version 6 packets transmitted.
     */
   card32 PacketsICMP6;

   /**
     * Number of raw bytes grouped by QoS class.
     */
   card64 ClassBytes[TrafficClassValues::MaxValues];

   /**
     * Number of packets grouped by QoS class.
     */
   card32 ClassPackets[TrafficClassValues::MaxValues];
};


/**
  * Implementation of << operator.
  */
ostream& operator<<(ostream& os, const NetworkMonitorReport& report);


}


#endif
