/*
 *  $Id: streamreport.h,v 1.2 2001/11/02 12:38:20 dreibh Exp $
 *
 * SCTP implementation according to RFC 2960.
 * Copyright (C) 1999-2001 by Thomas Dreibholz
 *
 * Realized in co-operation between Siemens AG
 * and University of Essen, Institute of Computer Networking Technology.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * There are two mailinglists available at www.sctp.de which should be used for
 * any discussion related to this implementation.
 *
 * Contact: discussion@sctp.de
 *          dreibh@exp-math.uni-essen.de
 *
 * Purpose: Stream Monitor Report
 *
 */


#ifndef STREAMREPORT_H
#define STREAMREPORT_H


#include "tdsystem.h"
#include "synchronizable.h"
#include "internetaddress.h"
#include "trafficclassvalues.h"


namespace Coral {


/**
  * This class contains a report for a single stream.
  *
  * @short   Stream Report
  * @author  Thomas Dreibholz (dreibh@exp-math.uni-essen.de)
  * @version 1.0
  *
  * @see StreamManagerReport
  */
struct StreamReport
{
   // ====== Constructor ====================================================
   /**
     * Constructor.
     */
   StreamReport();

   // ====== Byte order translation =========================================
   /**
     * Translate byte order.
     */
   void translate();

   // ====== Reset ==========================================================
   /**
     * Reset.
     */
   void reset();


   // ====== Comparision operators ==========================================
   /**
     * Implementation of == operator.
     */
   int operator==(const StreamReport& report) const;

   /**
     * Implementation of == operator.
     */
   int operator!=(const StreamReport& report) const;


   // ====== Values =========================================================
   public:
   /**
     * Source address of the stream.
     */
   PortableAddress Source;

   /**
     * Destination address of the stream.
     */
   PortableAddress Destination;

   /**
     * Flow label.
     */
   card32 FlowLabel;

   /**
     * Stream ID (SCTP only).
     */
   card32 StreamID;

   /**
     * Layer 3 Protocol number.
     */
   card8 Protocol3;

   /**
     * Layer 3 Protocol number.
     */
   card8 Protocol4;

   /**
     * Unused.
     */
   card8 pad[2];
   
   /**
     * Raw bytes sent in each traffic class.
     */
   card64 BytesRaw[TrafficClassValues::MaxValues];

   /**
     * Payload bytes sent in each traffic class.
     */
   card64 BytesPayload[TrafficClassValues::MaxValues];

   /**
     * Packets sent in traffic class.
     */
   card32 Packets[TrafficClassValues::MaxValues];
};


/**
  * Implementation of << operator.
  */
ostream& operator<<(ostream& os, const StreamReport& report);


}


#endif
