/*
 *  $Id: sctp_params.h,v 1.4 2001/11/23 16:13:26 ajung Exp $
 *
 * SCTP implementation according to RFC 2960.
 * Copyright (C) 2000 by Siemens AG, Munich, Germany.
 *
 * Realized in co-operation between Siemens AG
 * and University of Essen, Institute of Computer Networking Technology.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * There are two mailinglists available at http://www.sctp.de which should be
 * used for any discussion related to this implementation.
 *
 * Contact: discussion@sctp.de
 *          Michael.Tuexen@icn.siemens.de
 *          ajung@exp-math.uni-essen.de
 *
 * parameters for an association, and traffic_tool programs
 *
 */


#ifndef SCTP_PARAMS_H
#define SCTP_PARAMS_H

#include <sys/time.h>         /* for timeval etc. under Linux */
#include "sctp_wrapper.h"

#include <math.h>
#include <unistd.h>
#include <string.h>
#include <stdio.h>
#include <stdlib.h>         /* for atoi() under Linux */

#define MAX_RANDOM                            0x7fffffff
#define TRAFFIC_PORT                          1024
#define MAXIMUM_NUMBER_OF_LOCAL_ADDRESSES     4
#define MAXIMUM_NUMBER_OF_ASSOCIATIONS        2
#define MAXIMUM_NUMBER_OF_STREAMS             32
#define MAXIMUM_PAYLOAD_LENGTH                8192


#define min(x,y)            (x)<(y)?(x):(y)
FILE* fptr;
/* data for the traffic profile */
static unsigned short chunkLength  = 500;
static int numOfChunks      = 10;
static double param1        = 0.1;
static double param2        = 0.02;  /* min */
static double param3        = 100.0; /* max */

static struct timeval receiver_start_time;
static struct timeval sender_start_time;
static struct timeval shutdown_time;

/* parameters for the established associations */

static unsigned int InitialRTO  = 100;  /* 4*RTT = 80 */
/** the minimum timeout value */
static unsigned int  MinRTO = 40;  /* 2*RTT = 40 */
/** the maximum timeout value */
static unsigned int  MaxRTO = 500; /* 10 * RTT = 200 */
/** maximum retransmissions per association */
static unsigned int  AssocRetransmitLimit = 5;
/** maximum retransmissions per path */
static unsigned int  PathRetransmitLimit = 3;
/** maximum initial retransmissions */
static unsigned int  InitialRetransmitLimit = 5;
/** recvcontrol: delay for delayed ACK in msecs */
static unsigned int  SackDelay = 100;
/* let me have an unlimited queue */
static unsigned int UnlimitedQueueLength = 0;

static double delaysum             =0;
static int verbose                            = 0;
static int vverbose                           = 0;

/* user payload data structure */
struct c_data {
    struct timeval send_time;
    unsigned long int send_number;
};



struct ulp_data {
    int maximumStreamID;
    unsigned int assocID;
    unsigned long nrOfReceivedChunks;
    unsigned long nrOfReceivedBytes;
    unsigned long nrOfSentChunks;
    unsigned long nrOfSentChunksTotal;
    unsigned long nrOfSentBytes;
    unsigned long nrOfReceivedBytesPerStream[MAXIMUM_NUMBER_OF_STREAMS];
    unsigned long nrOfReceivedChunksPerStream[MAXIMUM_NUMBER_OF_STREAMS];
    unsigned long nrOfSentBytesPerStream[MAXIMUM_NUMBER_OF_STREAMS];
    unsigned long nrOfSentChunksPerStream[MAXIMUM_NUMBER_OF_STREAMS];
};


void dataArriveNotif(unsigned int assocID, unsigned int streamID, unsigned int len,
                     unsigned int protoID, unsigned int unordered, void* ulpDataPtr)
{
    unsigned char chunk[MAXIMUM_PAYLOAD_LENGTH];
    int length, index=0, result;
    struct timeval sender_delta_time;
    struct timeval receiver_delta_time;
    struct timeval current_time;
    struct timeval temp_time;
    struct timeval relative_time; /* time to shutdown */
    int udelay, mreltime;
    double mean;
    struct c_data* cd = (struct c_data*)chunk;

    if (vverbose) {
      fprintf(fptr, "%-8x: Data arrived (%u bytes on stream %u, %s)\n",
                      assocID, len, streamID, (unordered==SCTP_ORDERED_DELIVERY)?"ordered":"unordered");
      fflush(fptr);
    }
    /* read it */
    length = sizeof(chunk);
    SCTP_receive(assocID, streamID, chunk, &length, SCTP_MSG_DEFAULT);

    /* update counters */
    ((struct ulp_data *) ulpDataPtr)->nrOfReceivedChunks += 1;
    ((struct ulp_data *) ulpDataPtr)->nrOfReceivedBytes  += length;
    ((struct ulp_data *) ulpDataPtr)->nrOfReceivedChunksPerStream[streamID] += 1;
    ((struct ulp_data *) ulpDataPtr)->nrOfReceivedBytesPerStream[streamID]  += length;

    if (cd->send_number == 0) {
        /* do timer initialization */
        gettimeofday(&receiver_start_time,NULL);
        shutdown_time.tv_sec =  receiver_start_time.tv_sec+10;
        shutdown_time.tv_usec =  receiver_start_time.tv_usec;

        sender_start_time = cd->send_time;
        fprintf(fptr, "%-8x: Data Arrive Notification - initialized start times !\n", assocID);
        fprintf(fptr, "Now: %lu secs, %lu usecs - Peer: %lu secs, %lu usecs\n",
            receiver_start_time.tv_sec, receiver_start_time.tv_usec,
            cd->send_time.tv_sec, cd->send_time.tv_usec);

        fflush(fptr);
    } else { /* do the logging ! */
        gettimeofday(&current_time,NULL);
        timersub(&current_time, &receiver_start_time, &receiver_delta_time);
        // delay = receiver_delta_time.tv_sec*1000 + receiver_delta_time.tv_usec/1000;
        // fprintf(fptr, "Chunk %lu - Delta Receiver: Secs %lu - usecs %lu: %d msecs\n",
        //               cd->send_number,receiver_delta_time.tv_sec,receiver_delta_time.tv_usec, delay);

        timersub(&(cd->send_time), &sender_start_time, &sender_delta_time);
        // delay = sender_delta_time.tv_sec*1000 + sender_delta_time.tv_usec/1000;
        // fprintf(fptr, "Chunk %lu - Delta Sender: Secs %lu - usecs %lu: %d msecs\n",
        //               cd->send_number,sender_delta_time.tv_sec,sender_delta_time.tv_usec, delay);

        if (timercmp(&receiver_delta_time,&sender_delta_time, > ))  {
            timersub(&receiver_delta_time,&sender_delta_time, &temp_time);
            udelay = temp_time.tv_sec * 1000000 + temp_time.tv_usec;
            // fprintf(fptr, "Abs. Delay: Secs %lu - usecs %lu: %d.%d msecs\n",
            //          temp_time.tv_sec,temp_time.tv_usec, delay/1000, delay%1000);

        } else {
             timersub(&sender_delta_time,&receiver_delta_time, &temp_time);
             udelay = (temp_time.tv_sec * 1000000 + temp_time.tv_usec) * (-1);
             // fprintf(fptr, "Abs. Delay: Secs %lu - usecs %lu: %d.%d msecs\n",
             //         temp_time.tv_sec,temp_time.tv_usec, delay/1000, delay%1000);

        }
        if (timercmp(&shutdown_time,&current_time, > ))  {
             fprintf(fptr, "Now-: Secs %lu - usecs %lu - ShDN: Secs: %lu - usecs: %lu\n",
                      current_time.tv_sec,current_time.tv_usec, shutdown_time.tv_sec, shutdown_time.tv_usec);

           timersub(&shutdown_time,&current_time, &temp_time);
            mreltime = (-1)* (temp_time.tv_sec * 1000 +  temp_time.tv_usec/1000);
        } else {
            timersub(&current_time,&shutdown_time, &temp_time);
            mreltime = (temp_time.tv_sec * 1000 +  temp_time.tv_usec/1000);
             fprintf(fptr, "Now+: Secs %lu - usecs %lu - ShDN: Secs: %lu - usecs: %lu\n",
                      current_time.tv_sec,current_time.tv_usec, shutdown_time.tv_sec, shutdown_time.tv_usec);
        }

        delaysum += udelay;
        mean = (double)(delaysum / cd->send_number);
        fprintf(fptr, "Time: %d |Chunk %lu | Abs. Delay %d msecs | Avg. Delay %f msecs\n", mreltime,cd->send_number, udelay/1000, mean/1000.0);
        fflush(fptr);

    }
}



#endif
