/*
 *
 * SCTP implementation according to RFC 2960.
 * Copyright (C) 2000 by Siemens AG, Munich, Germany.
 *
 * Realized in co-operation between Siemens AG
 * and University of Essen, Institute of Computer Networking Technology.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * There are two mailinglists available at http://www.sctp.de which should be
 * used for any discussion related to this implementation.
 *
 * Contact: discussion@sctp.de
 *          Michael.Tuexen@icn.siemens.de
 *          ajung@exp-math.uni-essen.de
 *
 * 
 * Purpose : State keeping mechianism for the daemon.
 * Authors : Eric Yeo, Bryan Lim , Sim Woon Chiat ICM N PG U SE D2
 */

#include <stdlib.h>
#include <stdio.h>
#include "glib.h"
#include "sctpd_state.h"


#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#if defined (LINUX)
 #include <string.h>
 #include <unistd.h>
 #include <stdlib.h>
#endif


/***********************DEFINITION OF CONSTANTS**************************/

/*******************DEFINITION OF DATA STRUCTURE*************************/

/* Data structure to keep each UA information */
typedef
struct UA_Info {
 
  unsigned short uaID;
  unsigned short udp_notif_port;
  unsigned short udp_resp_port;
  //  unsigned short sctp_port;
  //  unsigned short instanceID;
  unsigned short state;

} UAInfo;

/* Data structure to keep each association information */
typedef
struct Assoc_Info {
  
  int assocID;
  unsigned short uaID;
  unsigned short instanceID;
  
} AssocInfo;

/* Data structure to keep each sctp instance information */
typedef
struct Instance_Info {
  unsigned short instanceID;
  unsigned short sctp_port;
  unsigned short uaID;
  unsigned short state;
  
} InstanceInfo;


/* Hack data structure to keep track of killed assoc 
 * whose UA are still around 
*/
typedef
struct Dead_Assoc_Info {
  unsigned int assocID;
  unsigned short instanceID;
  unsigned short uaID; 
} DeadAssocInfo;


/************************DEFINITION OF VARIABLES **************************/
static unsigned short MAX_NO_OF_ASSOC = 100;
static unsigned short MAX_NO_OF_UA = 20;
static unsigned short MAX_NO_OF_INSTANCE = 50;
static unsigned short CURRENT_NO_OF_ASSOC = 0;
static unsigned short CURRENT_NO_OF_UA = 0;
static unsigned short CURRENT_NO_OF_INSTANCE = 0;

/******************FUNCTION DEFINITION 1st Level***************************/

/* Internal Linked Lists to store state */

GSList  *uaInfoList        = NULL;
GSList  *assocInfoList     = NULL;
GSList  *instanceInfoList  = NULL;
GSList  *deadAssocList     = NULL;

/* Print all Utilities */

void PrintAll_UAInfoList(){

  UAInfo *result;
  GSList *node;
  
  fprintf(stdout,"\n/*********************UAInfoList**********************/\n"); 
  /* Search */
  for (node = uaInfoList; node; node = node->next) {    
    result = (UAInfo *)node->data;
    fprintf(stdout,"\nUA ID is:%d\n",result->uaID); 
    fprintf(stdout,"UDP notif port is:%u\n",result->udp_notif_port); 
    fprintf(stdout,"UDP resp port is:%u\n",result->udp_resp_port); 
    fprintf(stdout,"State of UA is:%u\n",result->state); 
  }
  fprintf(stdout,"\n/******************UAInfoList-END*******************/\n\n");
}

void PrintAll_AssocInfoList(){

  AssocInfo *result;
  GSList *node;
  
  fprintf(stdout,"\n/********************AssocInfoList********************/\n"); 
  /* Search */
  for (node = assocInfoList; node; node = node->next) {    
    result = (AssocInfo *)node->data;
    fprintf(stdout,"\nAssoc ID is:%d\n",result->assocID); 
    fprintf(stdout,"UA ID is:%d\n",result->uaID); 
    fprintf(stdout,"Instance ID is:%u\n",result->instanceID); 
  }
  fprintf(stdout,"\n/****************AssocInfoList-END******************/\n\n");
}

void PrintAll_DeadAssocList(){

  DeadAssocInfo *result;
  GSList *node;
  
  fprintf(stdout,"\n/********************DeadAssocList********************/\n"); 
  /* Search */
  for (node = deadAssocList; node; node = node->next) {    
    result = (DeadAssocInfo *)node->data;
    fprintf(stdout,"\nAssoc ID is:%d\n",result->assocID); 
    fprintf(stdout,"UA ID is:%d\n",result->uaID); 
    fprintf(stdout,"Instance ID is:%u\n",result->instanceID); 
  }
  fprintf(stdout,"\n/****************DeadAssocList-END******************/\n\n");
}


void PrintAll_InstanceInfoList(){

  InstanceInfo *result;
  GSList *node;
  
  fprintf(stdout,"\n/******************InstanceInfoList*******************/\n"); 
  /* Search */
  for (node = instanceInfoList; node; node = node->next) {    
    result = (InstanceInfo *)node->data;
    fprintf(stdout,"\nInstance ID is:%u\n",result->instanceID); 
    fprintf(stdout,"UA ID is:%d\n",result->uaID); 
    fprintf(stdout,"SCTP port is:%u\n",result->sctp_port); 
    fprintf(stdout,"State of Instance is:%u\n",result->state); 
  }
  fprintf(stdout,"\n/****************InstanceInfoList-END***************/\n\n");
}


/* Utility functions that return the list of UA IDs */
 
int GetAll_uaID(GSList **ua_ID) {
  
  UAInfo *ua_info;
  GSList *node;
  unsigned int counter = 0;   
  
  /* Search */
  for (node = uaInfoList; node; node = node->next) {
    ua_info = (UAInfo *)node->data;
    *ua_ID = g_slist_append (*ua_ID, &ua_info->uaID);
    counter++;
  }
  if (counter == 0) {
    /* Not found */
    return -1;
  }
  else{
    return counter;
  }
}


/* Utility functions that return the list of Assoc IDs */

int GetAll_assocID(GSList **assoc_ID) {
  
  AssocInfo *assoc_info;
  GSList *node;
  unsigned int counter = 0;   
  
  /* Search */
  for (node = assocInfoList; node; node = node->next) {
    assoc_info = (AssocInfo *)node->data;
    *assoc_ID = g_slist_append (*assoc_ID, &assoc_info->assocID);
    counter++;
  }
  if (counter == 0) {
    /* Not found */
    return -1;
  }
  else{
    return counter;
  }
}


/* Utility functions that return the list of IDs */

int GetAll_instanceID(GSList **instance_ID) {
  
  InstanceInfo *instance_info;
  GSList *node;
  unsigned int counter = 0;   
  
  /* Search */
  for (node = instanceInfoList; node; node = node->next) {
    instance_info = (InstanceInfo *)node->data;
    *instance_ID = g_slist_append (*instance_ID, &instance_info->instanceID);
    counter++;
  }
  if (counter == 0) {
    /* Not found */
    return -1;
  }
  else{
    return counter;
  }
}


/* UA Information functions */

int ua_insert_ua_info(UAInfo *ua_info) {
    
    /* Insert the copy into the list */ 
    uaInfoList = g_slist_append (uaInfoList, ua_info);

    return 0;
}

UAInfo * ua_delete_ua_info(unsigned short ua_ID) {
 
  UAInfo *ua_info;
  GSList *node;
  
  /* find the entry */
  for (node = uaInfoList; node; node = node->next) {

    ua_info = (UAInfo *)node->data;

    if (ua_info->uaID == ua_ID) {

      /* Remove from list */
      uaInfoList = g_slist_remove(uaInfoList, ua_info);

      return ua_info;
    }
  } 
  /* Not found */
  return NULL;
}

int ua_search_ua_info(unsigned short constant, void *value, GSList **ua_ID) {
  
  UAInfo *ua_info;
  GSList *node;
  unsigned int counter = 0;   

  /* Search */
  for (node = uaInfoList; node; node = node->next) {

    ua_info = (UAInfo *)node->data;
    
    switch (constant) {
      case SEARCH_UA_ID:
      if (ua_info->uaID == *(unsigned short*)value) {
	*ua_ID = g_slist_append (*ua_ID, &ua_info->uaID);
	counter++;
      }
      break;
      /*
    case SEARCH_INSTANCE_ID:
      if (ua_info->instanceID == *(unsigned short*)value) {
	*ua_ID = g_slist_append (*ua_ID,  &ua_info->uaID);
	counter++;
      }
      break;
      */
    case SEARCH_UDP_NOTIF_PORT:
      if (ua_info->udp_notif_port == *(unsigned short*)value) {
	*ua_ID = g_slist_append (*ua_ID, &ua_info->uaID);
	counter++;
      }
      break;
      
    case SEARCH_UDP_RESP_PORT:
      if (ua_info->udp_resp_port == *(unsigned short*)value) {
	*ua_ID = g_slist_append (*ua_ID, &ua_info->uaID);
	counter++;
      }
      break;
      /*      
    case SEARCH_SCTP_PORT:
      if (ua_info->sctp_port == *(unsigned short*)value) {
	*ua_ID = g_slist_append (*ua_ID, &ua_info->uaID);
	counter++;
      }
      break;
      */
    case SEARCH_STATE:
      if (ua_info->state == *(unsigned short*)value) {
	*ua_ID = g_slist_append (*ua_ID, &ua_info->uaID);
	counter++;
      }
      break;
    }
  }
  if (counter == 0) {
    /* Not found */
    return -1;
  }
  else{
    return counter;
  }
}

int ua_get_ua_info(unsigned short ua_ID, UAInfo **ua_info) {
  
  UAInfo *result;
  GSList *node;
  
  /* Search */
  for (node = uaInfoList; node; node = node->next) {
    
    result = (UAInfo *)node->data;
    if (result->uaID == ua_ID) {
      *ua_info = result;
      return 0;
    }
  }
  
  /* Not found */
  return -1;
}


/* Association Information functions */

int ua_insert_assoc_info(AssocInfo *assoc_info) {
    
    /* Insert the copy into the list */
    assocInfoList = g_slist_append (assocInfoList, assoc_info);
    
    return 0;
}

AssocInfo * ua_delete_assoc_info(unsigned int assoc_ID) {
 
  AssocInfo *assoc_info;
  GSList *node;
  
  /* find the entry */
  for (node = assocInfoList; node; node = node->next) {

    assoc_info = (AssocInfo *)node->data;

    if (assoc_info->assocID == assoc_ID) {

      /* Remove from list */
      assocInfoList = g_slist_remove(assocInfoList, assoc_info);

      return assoc_info;
    }
  } 
  /* Not found */
  return NULL;
}

int ua_search_assoc_info(unsigned short constant, void *value, 
			 GSList **assoc_ID) {
  
  AssocInfo *assoc_info;
  GSList *node;
  unsigned int counter = 0;
  
  /* Search */
  for (node = assocInfoList; node; node = node->next) {
    
    assoc_info = (AssocInfo *)node->data;
    
    switch (constant) {
      /*  case SEARCH_ASSOC_ID:
      if (assoc_info->assocID == *(unsigned int*)value) {
	*assoc_ID = g_slist_append (*assoc_ID, &assoc_info->assocID);
	counter++;
      }
      break; */
      
    case SEARCH_UA_ID:
      if (assoc_info->uaID == *(unsigned short*)value) {
	*assoc_ID = g_slist_append (*assoc_ID, &assoc_info->assocID);
	counter++;
      }
      break;

    case SEARCH_INSTANCE_ID:
      if (assoc_info->instanceID == *(unsigned short*)value) {
	*assoc_ID = g_slist_append (*assoc_ID, &assoc_info->assocID);
	counter++;
      }
      break;
    }
  }
  
  if (counter == 0) {
    /* Not found */
    return -1;
  }
  else{
    return counter;
  }
}

int ua_get_assoc_info(unsigned int assoc_ID, AssocInfo **assoc_info) {
  
  AssocInfo *result;
  GSList *node;
  
  /* Search */
  for (node = assocInfoList; node; node = node->next) {
    
    result = (AssocInfo *)node->data;
    if (result->assocID == assoc_ID) {
      *assoc_info = result;
      return 0;
    }
  }
  /* Not found */
  return -1;
}

/* Hack functions to handle dead associations */
int ua_insert_dead_assoc(DeadAssocInfo *dead_assoc) {
    
  /* Insert the copy into the list */
  deadAssocList = g_slist_append (deadAssocList, dead_assoc);
  
  return 0;
}

DeadAssocInfo * ua_delete_dead_assoc(unsigned int assoc_ID) {
 
  DeadAssocInfo *dead_assoc;
  GSList *node;
  
  /* find the entry */
  for (node = deadAssocList; node; node = node->next) {

    dead_assoc = (DeadAssocInfo *)node->data;

    if (dead_assoc->assocID == assoc_ID) {

      /* Remove from list */
      deadAssocList = g_slist_remove(deadAssocList, dead_assoc);

      return dead_assoc;
    }
  } 
  /* Not found */
  return NULL;
}

int ua_search_dead_assoc(unsigned short constant, void *value, 
			 GSList **assoc_ID) {
  
  DeadAssocInfo *dead_assoc;
  GSList *node;
  unsigned int counter = 0;
  
  /* Search */
  for (node = deadAssocList; node; node = node->next) {
    
    dead_assoc = (DeadAssocInfo *)node->data;
    
    switch (constant) {
    case SEARCH_INSTANCE_ID:
      if (dead_assoc->instanceID == *(unsigned short*)value) {
	*assoc_ID = g_slist_append (*assoc_ID, &dead_assoc->assocID);
	counter++;
      }
      break;    
    case SEARCH_UA_ID:
      if (dead_assoc->uaID == *(unsigned short*)value) {
	*assoc_ID = g_slist_append (*assoc_ID, &dead_assoc->assocID);
	counter++;
      }
      break;
    }
  }
  
  if (counter == 0) {
    /* Not found */
    return -1;
  }
  else{
    return counter;
  }
}

int ua_get_dead_assoc(unsigned int assoc_ID, DeadAssocInfo **dead_assoc) {
  
  DeadAssocInfo *result;
  GSList *node;
  
  /* Search */
  for (node = deadAssocList; node; node = node->next) {
    
    result = (DeadAssocInfo *)node->data;
    if (result->assocID == assoc_ID) {
      *dead_assoc = result;
      return 0;
    }
  }
  /* Not found */
  return -1;
}



/* Instance Information functions */

int instance_insert_info(InstanceInfo *instance_info) {

  /* Insert the copy into the list */ 
  instanceInfoList = g_slist_append (instanceInfoList, instance_info);
  
  return 0;
}

InstanceInfo * instance_delete_info(unsigned short instance_ID) {
 
  InstanceInfo *instance_info;
  GSList *node;
  
  /* find the entry */
  for (node = instanceInfoList; node; node = node->next) {
    instance_info = (InstanceInfo *)node->data;
    if (instance_info->instanceID == instance_ID) {
      /* Remove from list */
      instanceInfoList = g_slist_remove(instanceInfoList, instance_info);
      return instance_info;
    }
  } 
  /* Not found */
  return NULL;
}


int instance_search_info(unsigned short constant, void *value,
			 GSList **instance_ID) {
  
  InstanceInfo *instance_info;
  GSList *node;
  unsigned int counter = 0;   

  /* Search */
  for (node = instanceInfoList; node; node = node->next) {

    instance_info = (InstanceInfo *)node->data;
    
    switch (constant) {
    case SEARCH_INSTANCE_ID:
      if (instance_info->instanceID == *(unsigned short*)value) {
	*instance_ID = g_slist_append (*instance_ID, &instance_info->instanceID);
	counter++;
      }
      break;

    case SEARCH_UA_ID:
      if (instance_info->uaID == *(unsigned short*)value) {
	*instance_ID = g_slist_append (*instance_ID, &instance_info->instanceID);
	counter++;
      }
      break;
            
    case SEARCH_SCTP_PORT:
      if (instance_info->sctp_port == *(unsigned short*)value) {
	*instance_ID = g_slist_append (*instance_ID, &instance_info->instanceID);
	counter++;
      }
      break;
    }
  }
  if (counter == 0) {
    /* Not found */
    return -1;
  }
  else{
    return counter;
  }
}

int instance_get_info(unsigned short instance_ID, 
		      InstanceInfo **instance_info) {
  
  InstanceInfo *result;
  GSList *node;
  
  /* Search */
  for (node = instanceInfoList; node; node = node->next) {    
    result = (InstanceInfo *)node->data;
    if (result->instanceID == instance_ID) {
      *instance_info = result;
      return 0;
    }
  }

  /* Not found */
  return -1;
}



/********************Function definition 2nd level*********************/

/*  This function returns 0 if successful,
 *  -1 if otherwise.
 */
int state_add_ua (unsigned short ua_id, unsigned short udp_notif_port, 
		  unsigned  short udp_resp_port) {
  UAInfo *ua_info;
  int result;

  if ((ua_info = (UAInfo *)malloc(sizeof(UAInfo)))!=NULL) {
    ua_info->uaID = ua_id;
    ua_info->udp_notif_port = udp_notif_port;
    ua_info->udp_resp_port = udp_resp_port;
    ua_info->state = UA_ACTIVE_STATE;
    result = ua_insert_ua_info(ua_info);
  }
  else {
    result=-1;
#ifdef DEBUG
    printf("Error in allocating memory for instance\n");
#endif
  }
    
  if (result == 0)
    return result;
  else
    return -1;

}


/*  This function returns 0 if successful,
 *  -1 if otherwise.
 */
int state_remove_ua (unsigned short ua_id){
  
  UAInfo *ua_info;
 
  if ((ua_info = ua_delete_ua_info(ua_id))!=NULL){
    free(ua_info);
    return 0;
  }
  else
    return -1;
}


/*  This function returns 0 if successful,
 *  -1 if otherwise.
 */
int state_add_assoc (unsigned int assocID, 
		     unsigned short ua_id,
		     unsigned short instanceID){
 
  int result;
  AssocInfo *assoc_info;

  if ((assoc_info = (AssocInfo *)malloc(sizeof(AssocInfo)))!=NULL) {
    assoc_info->assocID = assocID;
    assoc_info->uaID = ua_id;
    assoc_info->instanceID = instanceID;
    result = ua_insert_assoc_info(assoc_info);
  }
  else {
    result=-1;
#ifdef DEBUG
    printf("Error in allocating memory for instance\n");
#endif
  }

  if (result == 0)
    return result;
  else
    return -1;
}


/*  This function returns 0 if successful,
 *  -1 if otherwise.
 */
int state_remove_assoc (unsigned int assocID){
 
  AssocInfo *assoc_info;
  
  if ((assoc_info = ua_delete_assoc_info(assocID))!=NULL){
    free(assoc_info);
    return 0;
  }
  else
    return -1;
}

/*  This function returns 0 if successful,
 *  -1 if otherwise.
 */
int state_add_dead_assoc (unsigned int assocID, 
			  unsigned short ua_id,
			  unsigned short instanceID){
 
  int result;
  DeadAssocInfo *assoc_info;

  if ((assoc_info = (DeadAssocInfo *)malloc(sizeof(DeadAssocInfo)))!=NULL) {
    assoc_info->assocID = assocID;
    assoc_info->uaID = ua_id;
    assoc_info->instanceID = instanceID;
    result = ua_insert_dead_assoc(assoc_info);
  }
  else {
    result=-1;
#ifdef DEBUG
    printf("Error in allocating memory for instance\n");
#endif
  }

  if (result == 0)
    return result;
  else
    return -1;
}


/*  This function returns 0 if successful,
 *  -1 if otherwise.
 */
int state_remove_dead_assoc (unsigned int assocID){
 
  DeadAssocInfo *assoc_info;
  
  if ((assoc_info = ua_delete_dead_assoc(assocID))!=NULL){
    free(assoc_info);
    return 0;
  }
  else
    return -1;
}


/*  This function returns 0 if successful,
 *  -1 if otherwise.
 */
int state_add_instance (unsigned short instanceID, 
			unsigned short sctp_port, 
			unsigned  short ua_id) {
  InstanceInfo *instance_info;
  int result;

  if ((instance_info = (InstanceInfo *)malloc(sizeof(InstanceInfo)))!=NULL) {
    instance_info->instanceID = instanceID;
    instance_info->uaID = ua_id;
    instance_info->sctp_port = sctp_port;
    instance_info->state = INSTANCE_ACTIVE_STATE;
    result = instance_insert_info(instance_info);
  }
  else {
    result=-1;
#ifdef DEBUG
    printf("Error in allocating memory for instance\n");
#endif
  }
     
  if (result == 0){
    return result;
  }
  else
    return -1;

}


/*  This function returns 0 if successful,
 *  -1 if otherwise.
 */
int state_remove_instance (unsigned short ua_id){
  
  InstanceInfo *instance_info;
 
  if ((instance_info = instance_delete_info(ua_id))!=NULL){
    free(instance_info);
    return 0;
  }
  else
    return -1;
}


/*  This function returns the no. of associations found if successful,
 *  -1 if no associations are found.
 */
int state_uaID_to_assoc_list (unsigned short ua_id, GSList **assocID_list){

  guint result;

  result = ua_search_assoc_info(SEARCH_UA_ID,(void *)&ua_id, assocID_list);
  if (result != -1)
    return result;
  else
    return -1;
}


/*  This function returns the no. of associations found if successful,
 *  -1 if no associations are found.
 */
int state_instanceID_to_assoc_list (unsigned short instance_id, 
				    GSList **assocID_list){

  guint result;

  result = ua_search_assoc_info(SEARCH_INSTANCE_ID,(void *)&instance_id, 
				assocID_list);
  if (result != -1)
    return result;
  else
    return -1;
}


/*  This function returns the no. of instances found if successful,
 *  -1 if no associations are found.
 */
int state_uaID_to_instance_list (unsigned short ua_id, GSList **instanceID_list){

  guint result;

  result = instance_search_info(SEARCH_UA_ID,(void *)&ua_id, instanceID_list);
  if (result != -1)
    return result;
  else
    return -1;
}


/*  This function returns 0 if successful,
 *  -1 if otherwise.
 */
int state_assocID_to_ua_id (unsigned int assocID, unsigned short* ua_id){

  int result;
  AssocInfo *assoc_info;
  DeadAssocInfo *dead_assoc_info;

  result = ua_get_assoc_info(assocID, &assoc_info);
  if (result == 0){
    *ua_id = assoc_info->uaID;    
    return result;
  }
  else {
    /* HACK : Check the dead list */
#ifdef DEBUG
    fprintf(stdout,"Looking in dead list\n");
    fflush(stdout);
#endif
     result = ua_get_dead_assoc(assocID, &dead_assoc_info);
     if (result == 0){
#ifdef DEBUG
       fprintf(stdout,"Assoc found in dead list\n");
       fflush(stdout);
#endif
       *ua_id = dead_assoc_info->uaID;          
       return result;
     }   
  }

  return -1;
}

/*  This function returns 0 if successful,
 *  -1 if otherwise.
 */
int state_assocID_to_instanceID (unsigned int assocID, unsigned short* instanceID){

  int result;
  AssocInfo *assoc_info;
  DeadAssocInfo *dead_assoc_info;

  result = ua_get_assoc_info(assocID, &assoc_info);
  if (result == 0){
    *instanceID = assoc_info->instanceID;    
    return result;
  }
  else{
    /* HACK : Check the dead list */
     result = ua_get_dead_assoc(assocID, &dead_assoc_info);
     if (result == 0){
       *instanceID = assoc_info->instanceID;    
       return result;
     }   
  }
  return -1;
}


/*  This function returns 0 if successful,
 *  -1 if otherwise.
 */
int state_instanceID_to_ua_id (unsigned short instanceID, unsigned short* ua_id){

  int result;
  InstanceInfo *instance_info;

  result = instance_get_info(instanceID, &instance_info);
  if (result == 0){
    *ua_id = instance_info->uaID;    
    return result;
  }
  else
    return -1;

}


/*  This function returns the 0 if successful,
 *  -1 if otherwise.
 */
int state_get_udp_notif_port (unsigned short ua_id, 
			      unsigned short* udp_notif_port){

  int result;
  UAInfo *ua_info;

  result = ua_get_ua_info(ua_id, &ua_info);
  if (result == 0){
    *udp_notif_port = ua_info->udp_notif_port;
    return result;
  }
  else
    return -1;
}


/*  This function returns 0 if successful,
 *  -1 if otherwise.
 */
int state_get_udp_resp_port (unsigned short ua_id, 
			     unsigned short* udp_resp_port){

  int result;
  UAInfo *ua_info;

  *udp_resp_port = 0;

  result = ua_get_ua_info(ua_id, &ua_info);
  if (result == 0){
    *udp_resp_port = ua_info->udp_resp_port;
    return result;
  }
  else
    return -1;
}


/*  This function returns 0 if successful,
 *  -1 if otherwise.
 */
int state_get_sctp_port (unsigned short instanceID, unsigned short* sctp_port){

  int result;
  InstanceInfo *instance_info;

  result = instance_get_info(instanceID, &instance_info);
  if (result == 0){
    *sctp_port = instance_info->sctp_port;
    return result;
  }
  else
    return -1;
}


/*  This function returns 0 if successful,
 *  -1 if otherwise.
 */
int state_set_sctp_port (unsigned short instanceID, unsigned short sctp_port){

  int result;
  InstanceInfo *instance_info;

  result = instance_get_info(instanceID, &instance_info);
  if (result == 0){
    instance_info->sctp_port = sctp_port;
    return result;
  }
  else
    return -1;
}


/*  This function returns the state of the UA if successful,
 *  -1 if otherwise.
 */
int state_get_instance_state (unsigned short instance_id){

  int result;
  InstanceInfo *instance_info;

  result = instance_get_info(instance_id, &instance_info);
  if (result == 0){
    result = instance_info->state;
    return result;
  }
  else
    return -1;
}


/*  This function returns 0 if successful,
 *  -1 if otherwise.
 */
int state_set_instance_state (unsigned short instance_id, unsigned short state){

  int result;
  InstanceInfo *instance_info;

  result = instance_get_info(instance_id, &instance_info);
  if (result == 0){
    instance_info->state = state;
    return result;
  }
  else
    return -1;
}



/*  This function returns the state of the UA if successful,
 *  -1 if otherwise.
 */
int state_get_ua_state (unsigned short ua_id){

  int result;
  UAInfo *ua_info;

  result = ua_get_ua_info(ua_id, &ua_info);
  if (result == 0){
    result = ua_info->state;
    return result;
  }
  else
    return -1;
}


/*  This function sets the state of the UA and all instances belonging to it to
 *  zombie state. Returns 0 if successful,
 *  -1 if otherwise.
 */
int state_set_ua_state (unsigned short ua_id, unsigned short state){

  int result, i;
  unsigned short instanceID;
  guint length;
  UAInfo *ua_info;
  GSList *instanceID_list = NULL;

  result = ua_get_ua_info(ua_id, &ua_info);
  if (result == 0){
    ua_info->state = state;
    if (instance_search_info(SEARCH_UA_ID,(void *)&ua_id, &instanceID_list)!=-1){
      length = g_slist_length (instanceID_list);
      for (i =0; i<length; i++){
	instanceID = * (unsigned short *)(g_slist_nth_data (instanceID_list, i));
	if (state_set_instance_state (instanceID, state)== 0){
#ifdef DEBUG
	  fprintf(stdout,"State of instance ID:%d is now zombie.\n", instanceID);
	  fflush(stdout);
#endif
	}
      }
    }
    /* free the glist */
    g_slist_free(instanceID_list);
    return result;
  }
  return -1;
}


/*  This function returns 0 if UA exists,
 *  -1 if otherwise.
 */
int state_check_ua_exist (unsigned short ua_id){

  int result;
  GSList *ua_list = NULL;

  result = ua_search_ua_info(SEARCH_UA_ID,(void *)&ua_id, &ua_list);
  g_slist_free(ua_list);

  if (result != -1)
    return 0;
  else
    return -1;
}


/*  This function returns 0 if Instance exists,
 *  -1 if otherwise.
 */
int state_check_instance_exist (unsigned short instance_id){

  int result;
  GSList *instance_list = NULL;

  result = instance_search_info(SEARCH_INSTANCE_ID,(void *)&instance_id, &instance_list);
  g_slist_free(instance_list);

  if (result != -1)
    return 0;
  else
    return -1;
}


/*  This function returns 0 if the SCTP port is used,
 *  -1 if otherwise.
 */ 
int state_check_sctp_port_used (unsigned short sctp_port){

  int result;
  GSList *instance_list = NULL;

  result = instance_search_info(SEARCH_SCTP_PORT, (void *)&sctp_port,
				&instance_list);
  g_slist_free(instance_list);

  if (result != -1)
    return 0;
  else
    return -1;
}

/*  This function returns 0 if UDP notification port is used,
 *  -1 if otherwise.
 */
int state_check_udp_notif_port_used (unsigned short udp_notif_port){

  int result;
  GSList *ua_list = NULL;

  result = ua_search_ua_info(SEARCH_UDP_NOTIF_PORT, (void *)&udp_notif_port, 
			  &ua_list);
  g_slist_free(ua_list);
  if (result != -1)
    return 0;
  else
    return -1;
}


/*  This function returns 0 if UDP response port is used,
 *  -1 if otherwise.
 */
int state_check_udp_resp_port_used (unsigned short udp_resp_port){

  int result;
  GSList *ua_list = NULL;

  result = ua_search_ua_info(SEARCH_UDP_NOTIF_PORT, (void *)&udp_resp_port, 
			     &ua_list);
  g_slist_free(ua_list);
  if (result != -1)
    return 0;
  else
    return -1;
}





 
/********************Function definition 3rd level*********************/


/*  This function is a helper function to retrieve the next remaining assoc ID
 *  belonging to a Zombie UA so as to call sctp_shutdown for all remaining 
 *  associations. 
 *  In making this function call, use a while loop so that the counter is reset
 *  when there are no more association in the list. 
 *  This function returns the next association ID in the GSList if it exists, 
 *  -1 if otherwise.
 */
int sctpd_state_get_next_assocID (GSList *assocID_list ){

  int result;
  static unsigned short element = 0;
  guint length;

  length = g_slist_length (assocID_list);
  if (element < length){
    result = *(int *) (g_slist_nth_data (assocID_list, element));
    element++;
  }
  else{
    /* reset counter */
    element = 0;
#ifdef DEBUG
    // fprintf(stdout, "nth element reset for assocID List:%d\n", element);
#endif
    result = -1;
  }
  return result;
}


/*  This function is a helper function to retrieve the next remaining instance
 *  ID belonging to a Zombie UA so as to call sctp_UnregisterInstance for all  
 *  remaining associations. 
 *  In making this function call, use a while loop so that the counter is reset
 *  when there are no more instance ID entry in the list. 
 *  This function returns the next instance ID in the GSList if it exists, 
 *  -1 if otherwise.
 */
short sctpd_state_get_next_instanceID (GSList *instanceID_list ){

  short result;
  static unsigned short nth_element = 0;
  guint length;

  length = g_slist_length (instanceID_list);

  if (nth_element < length){
    result = * (short *)(g_slist_nth_data (instanceID_list, nth_element));
    nth_element++;
  }
  else{
    /* reset counter */
    nth_element = 0;
    result = -1;
#ifdef DEBUG
  // fprintf(stdout, "nth element reset for InstanceID List:%d\n", nth_element);
#endif
  }
 return result;
}

short sctpd_state_get_next_uaID (GSList *uaID_list ){

  short result;
  static unsigned short ua_element = 0;
  guint length;

  length = g_slist_length (uaID_list);

  if (ua_element < length){
    result = * (short *)(g_slist_nth_data (uaID_list, ua_element));
    ua_element++;
  }
  else{
    /* reset counter */
    ua_element = 0;
    result = -1;
#ifdef DEBUG
  // fprintf(stdout, "nth element reset for uaID List:%d\n", ua_element);
#endif
  }
 return result;
}



int sctpd_state_register_ua (unsigned short ua_id, 
			     unsigned short udp_notif_port, 
			     unsigned short udp_resp_port, short force){
  int result;

  /* if force registration is not true */
  if (force == 0 && CURRENT_NO_OF_UA < MAX_NO_OF_UA ){
    /* Check if the UA is a duplicate by searching if the udp_resp_port is used,
       the function returns -1 if the udp_resp_port is not used */
    if ((state_check_udp_resp_port_used(udp_resp_port)) == -1){
      result = state_add_ua(ua_id, udp_notif_port, udp_resp_port);
      if (result == 0){
	CURRENT_NO_OF_UA++;
#ifdef DEBUG       
	fprintf(stdout, "UA id:%d registered!\n", ua_id);
#endif
	return result;
      }
      else {

#ifdef DEBUG       
	fprintf(stdout, "Registering UA id:%d failed.\n", ua_id);
#endif
	return result;
      }
    }
    else {

#ifdef DEBUG       
      fprintf(stdout,"Duplicated UA id:%d, registration fails.\n", ua_id);
#endif

      return 1;
    }
  }
  else if (force == 1 && CURRENT_NO_OF_UA < MAX_NO_OF_UA) {
    result = state_add_ua(ua_id, udp_notif_port, udp_resp_port); 
    if (result == 0){
      CURRENT_NO_OF_UA++;
#ifdef DEBUG       
      fprintf(stdout, "UA id:%d registered!\n", ua_id);
#endif
      return result;
    }
    else if(result == -1) {
#ifdef DEBUG       
      fprintf(stdout, "Registering UA id:%d failed.\n", ua_id);
#endif
      return result;
    }
  }
  else if (CURRENT_NO_OF_UA == MAX_NO_OF_UA){
#ifdef DEBUG       
    fprintf(stdout,"Registering UA id:%d failed, MAX_NO_OF_UA reached\n",ua_id);
#endif
    return 2;
  }

  return -1;
}


int sctpd_state_unregister_ua (unsigned short ua_id, short force){

  int result;
  GSList *assocID_list = NULL;
  GSList *instanceID_list = NULL;

  if (state_check_ua_exist(ua_id) == 0){
    /* if there are also no outstanding instances */
    if (state_uaID_to_instance_list(ua_id, &instanceID_list) == -1){
      g_slist_free(instanceID_list);
      /* Check if there is outstanding association, if not,
	 proceed to remove the UA from List */
      if ((state_uaID_to_assoc_list(ua_id, &assocID_list)) == -1){
	/* free the glist */
	g_slist_free(assocID_list);
	result = state_remove_ua (ua_id);
	if (result == 0){
	  CURRENT_NO_OF_UA--;
#ifdef DEBUG       
	  fprintf(stdout,"UA id:%d deregistered.\n", ua_id);
	  fprintf(stdout,"Current no. of UA is:%d\n",CURRENT_NO_OF_UA);
#endif
	  return result;
	}
	else {
#ifdef DEBUG       
	  fprintf(stdout,"UA id:%d deregistering failed!\n", ua_id);
#endif
	  return -1;
	}
      }
      /* If there are outstanding assoc, return 1 */
      else {
	if (force == 1){
	  /* if force unregistration is true, state the state of UA to zombie */
	  if (state_set_ua_state (ua_id, UA_ZOMBIE_STATE) == 0){ 
#ifdef DEBUG
	    fprintf(stdout,"State of UA ID:%d is now Zombie!\n", ua_id);
#endif	  
	  }
	}
	/* free the glist */
	g_slist_free(assocID_list);
	return 1;
      }
    }
    /* if there are outstanding instances, */
    else {
      /* if force ==1, set the set of the UA to zombie */
      if (force == 1){
	if (state_set_ua_state (ua_id, UA_ZOMBIE_STATE) == 0){ 
#ifdef DEBUG
	  fprintf(stdout,"State of UA ID:%d is now Zombie!\n", ua_id);
#endif	  
	}   
      }
      /* free the glist */
      g_slist_free(instanceID_list);
      return 2;
    }
  }
  else{
#ifdef DEBUG
    fprintf(stdout,"UA not found or already deregistered.\n");
#endif
    return 3;
  }
  // return -1;
}


int sctpd_state_add_assoc (unsigned int assocID, 
			   unsigned short ua_id,
			   unsigned short instanceID){

  int result;
  
  if (CURRENT_NO_OF_ASSOC<MAX_NO_OF_ASSOC && 
      state_check_ua_exist(ua_id) == 0 && 
      state_get_ua_state(ua_id) == UA_ACTIVE_STATE && 
      state_check_instance_exist(instanceID) == 0 &&
      state_get_instance_state(instanceID) == INSTANCE_ACTIVE_STATE){
    
    result = state_add_assoc (assocID, ua_id,instanceID);
    if (result == 0){
      CURRENT_NO_OF_ASSOC++;	
#ifdef DEBUG
      fprintf(stdout,"Adding assoc ID:%d successful\n",assocID);
#endif
      return result;
    }
    else if (result == -1) {
#ifdef DEBUG
      fprintf(stdout,"Adding association failed!\n");
#endif
      return result;
    }	
  }
  else if (state_check_ua_exist(ua_id) == -1) {
#ifdef DEBUG
    fprintf(stdout,"UA not found, adding assoc ID:%d failed!\n", assocID);
#endif
    return 1;
  }
  else if (CURRENT_NO_OF_ASSOC == MAX_NO_OF_ASSOC){
#ifdef DEBUG
    fprintf(stdout,"MAX no. of assoc exceeded, adding assoc ID:%d failed!\n", assocID);
#endif
    return 2;
  }
  else if (state_get_ua_state(ua_id) == UA_ZOMBIE_STATE){
#ifdef DEBUG
    fprintf(stdout,"UA is in Zombie state, adding assoc ID:%d failed!\n", assocID);
#endif
    return 3;
  }
  else if (state_check_instance_exist(instanceID) == -1) {
#ifdef DEBUG
    fprintf(stdout,"Instance not found, adding assoc ID:%d failed!\n", assocID);
#endif
    return 4;
  }
  else if (state_get_instance_state(instanceID) == INSTANCE_ZOMBIE_STATE){
#ifdef DEBUG
    fprintf(stdout,"Instance is in Zombie state, adding assoc ID:%d failed!\n",instanceID);
#endif
    return 5;
  }

  return -1;

}

int sctpd_state_remove_assoc (unsigned int assocID){

  int t=-1;
  int result =-1;  
  GSList *assocID_list = NULL;
  unsigned short instance_id=0;
  unsigned short ua_id=0;
  
  /* Get the instance ID for this assoc */
  state_assocID_to_instanceID(assocID, &instance_id);
  /* Get the UA ID for this assoc */
  sctpd_state_assocID_to_ua_id (assocID, &ua_id);

  /* Remove the association */
  result = state_remove_assoc (assocID); 
  if (result == 0){
    CURRENT_NO_OF_ASSOC--;
#ifdef DEBUG
    fprintf(stdout,"Assoc ID:%d removed successfully.\n", assocID);
    fprintf(stdout,"Total association(s) left:%d\n", CURRENT_NO_OF_ASSOC);
#endif
  }
  else {
#ifdef DEBUG
    fprintf(stdout,"Removing association failed!\n");
#endif
  }

  if (instance_id!=0){

    /* Check if it is a zombie instance */
    if (state_get_instance_state(instance_id) == INSTANCE_ZOMBIE_STATE){  
      /* Instance is a zombie */
      /* if there are no more outstanding association belonging to the 
	 instance, remove the zombie Instance */
      if (state_instanceID_to_assoc_list (instance_id, &assocID_list) == -1){
	if (sctpd_state_remove_instance (instance_id)==0){
	  /* Assign the instance ID to the return value */
	  result = instance_id;
	}
      }
      /* free the glist */
      g_slist_free(assocID_list);
    }

    /* HACK: Add the assoc to the dead list */
    /* Get the UA id of this instance */
    if (ua_id!=0) {
      if ((t = state_add_dead_assoc(assocID,ua_id,instance_id))==0) {
#ifdef DEBUG
	fprintf(stdout,"Adding association to deal list successful!\n");
#endif
      }
      else {
#ifdef DEBUG
	fprintf(stdout,"Adding association to deal list Failed!\n");
#endif
      }
    }
  }
  
  return result;
}




int sctpd_state_add_instance (unsigned short instanceID,unsigned short sctp_port,
			      unsigned short ua_id) {

  int result;
  
  if (state_check_ua_exist(ua_id)==0 && 
      CURRENT_NO_OF_INSTANCE < MAX_NO_OF_INSTANCE &&
      state_get_ua_state(ua_id) == UA_ACTIVE_STATE){
    /* Check if the sctp_port is used,
       the function returns -1 if the sctp_port is not used */
    if ((state_check_sctp_port_used(sctp_port)) == -1 || 
	(sctp_port ==  0)){
      result = state_add_instance (instanceID, sctp_port,ua_id);
      if (result == 0){
	CURRENT_NO_OF_INSTANCE++;	
#ifdef DEBUG
	fprintf(stdout,"Adding instance ID: %d successful\n",instanceID);
#endif
	return result;
      }
      else if (result == -1) {
#ifdef DEBUG
	fprintf(stdout,"Adding instance failed!\n");
#endif
	return result;
      }	
    }
    /* if sctp port is used */
    else{
#ifdef DEBUG
      fprintf(stdout,"SCTP port is already used, adding instance failed!\n");
#endif
      return 4;
    }
  }
  else if (state_check_ua_exist(ua_id) == -1) {
#ifdef DEBUG
    fprintf(stdout,"UA not found, adding instance ID:%d failed!\n", instanceID);
#endif
    return 1;
  }
  else if (CURRENT_NO_OF_INSTANCE == MAX_NO_OF_INSTANCE){
#ifdef DEBUG
    fprintf(stdout,"MAX no. of instance exceeded, adding instance ID:%d failed!\n", instanceID);
#endif
    return 2;
  }
  else if (state_get_ua_state(ua_id) == UA_ZOMBIE_STATE){
#ifdef DEBUG
    fprintf(stdout,"UA is in Zombie state, adding instance ID:%d failed!\n", instanceID);
#endif
    return 3;
  }

  return -1;

}


int sctpd_state_remove_instance (unsigned int instanceID){

  int result;
  unsigned short ua_id;
  GSList *assocID_list = NULL;
  GSList *instanceID_list = NULL;
  
  /* check if instance exists */
  if (state_check_instance_exist(instanceID)==0){
    /* if there are no outstanding associations belonging to this instance */
    if (state_instanceID_to_assoc_list(instanceID, &assocID_list) == -1){
      g_slist_free(assocID_list);
      if (sctpd_state_instanceID_to_ua_id (instanceID, &ua_id) == 0){
	result = state_remove_instance(instanceID);
	/* check if parent UA is a Zombie */
	if (state_get_ua_state(ua_id) == UA_ZOMBIE_STATE){  
	  /* if there are no more outstanding instances belonging to the UA, 
	     remove the zombie UA */
	  if (state_uaID_to_instance_list (ua_id, &instanceID_list) == -1){
	    if (state_remove_ua (ua_id)==0){
	      CURRENT_NO_OF_UA--;	
#ifdef DEBUG
	      fprintf(stdout,"Zombie UA id:%d removed.\n", ua_id);
	      fprintf(stdout,"Current no. of UA is:%d\n",CURRENT_NO_OF_UA);
#endif
	    }
	  }
	  /* free the glist */
	  g_slist_free(instanceID_list);
	}
      }
      else{
	result = state_remove_instance(instanceID);
      }

      if (result == 0){
	CURRENT_NO_OF_INSTANCE--;
#ifdef DEBUG
	fprintf(stdout,"Instance ID:%d removed successfully.\n", instanceID);
	fprintf(stdout,"Total instance(s) left:%d\n", CURRENT_NO_OF_INSTANCE);
#endif
	return result;
      }
      else {
#ifdef DEBUG
	fprintf(stdout,"Removing Instance failed!\n");
#endif
	return -1;
      }	
    }
    /* if there are outstanding associations */
    else{
#ifdef DEBUG
      fprintf(stdout,"Outstanding associations, instance not removed!\n");
#endif
      /* Set the state of instance to zombie */
      if (state_set_instance_state(instanceID, INSTANCE_ZOMBIE_STATE)==0){
#ifdef DEBUG
	fprintf(stdout,"State of instance ID:%d is now Zombie!\n", instanceID);
#endif
      }
      /* free the glist */
      g_slist_free(assocID_list);
      return 1;
    }
  }
  /* instance does not exist */
  else{
#ifdef DEBUG
    fprintf(stdout,"Instance ID:%d does not exist\n", instanceID);
#endif
    return 2;
  }
  // return -1;
}


int sctpd_state_uaID_to_assoc_list (unsigned short ua_id, GSList **assocID_list){

  int result;

  result = state_uaID_to_assoc_list (ua_id, assocID_list);
  return result;  
}

int sctpd_state_instanceID_to_assoc_list (unsigned short instance_id, 
					  GSList **assocID_list){

  int result;

  result = state_instanceID_to_assoc_list (instance_id, assocID_list);
  return result;  
}

int sctpd_state_uaID_to_instance_list (unsigned short ua_id,
				       GSList **instanceID_list){

  int result;

  result = state_uaID_to_instance_list (ua_id, instanceID_list);
  return result;  

}


int sctpd_state_assocID_to_ua_id (unsigned int assocID, unsigned short* ua_id){

  int result;
  
  result = state_assocID_to_ua_id (assocID, ua_id);
  return result;  
}


int sctpd_state_assocID_to_instanceID (unsigned int assocID,
				       unsigned short* instanceID){

  int result;
  
  result = state_assocID_to_instanceID (assocID, instanceID);
  return result;  
}


int sctpd_state_instanceID_to_ua_id (unsigned short instanceID, 
				     unsigned short* ua_id){
 
  int result;
  
  result = state_instanceID_to_ua_id (instanceID, ua_id);
  return result;  
}


int sctpd_state_get_udp_notif_port (unsigned short ua_id,
				    unsigned short* udp_notif_port){

  int result;
  
  result = state_get_udp_notif_port (ua_id, udp_notif_port );
  return result;  
}


int sctpd_state_get_udp_resp_port (unsigned short ua_id,
				   unsigned short* udp_resp_port){

  int result;

  result = state_get_udp_resp_port (ua_id, udp_resp_port);  
  return result;
}


int sctpd_state_get_sctp_port (unsigned short instanceID, 
			       unsigned short* sctp_port){

  int result;
  
  result = state_get_sctp_port (instanceID, sctp_port);
  return result;
}


int sctpd_state_set_sctp_port (unsigned short instanceID, 
			       unsigned short sctp_port){
  
  int result;
  
  result = state_set_sctp_port (instanceID, sctp_port);
  return result;
}

/*  This function returns 0 if UA exists,
 *  -1 if otherwise.
 */
int sctpd_state_check_ua_exist (unsigned short ua_id){

  int result;
 
  result = state_check_ua_exist(ua_id);

  if (result != -1)
    return 0;
  else
    return -1;
}


int sctpd_state_set_max_no_of_ua (unsigned short max_no_of_ua){
  
  MAX_NO_OF_UA = max_no_of_ua;

  if (MAX_NO_OF_UA == max_no_of_ua)
    return 0;
  else 
    return -1;
}

int sctpd_state_set_max_no_of_assoc (unsigned short max_no_of_assoc){

  MAX_NO_OF_ASSOC = max_no_of_assoc;
  
  if (MAX_NO_OF_ASSOC == max_no_of_assoc)
    return 0;
  else 
    return -1;
}

int sctpd_state_set_max_no_of_instance (unsigned short max_no_of_instance){

  MAX_NO_OF_INSTANCE = max_no_of_instance;
  
  if (MAX_NO_OF_INSTANCE == max_no_of_instance)
    return 0;
  else 
    return -1;
}


/* Utility functions to get port numbers */
unsigned short getUDPNotifPort(int assocID) {

  unsigned short notifPort=0;
  unsigned short ua_id=0;

  /* Get the ua_id for this association first */
  if (sctpd_state_assocID_to_ua_id(assocID,&ua_id)==0) {
    /* Get the Notif Port number */
    if ((sctpd_state_get_udp_notif_port(ua_id,&notifPort)==0)) {
       return notifPort;
    }
  }
    
  return 0;

}


unsigned short getUDPRespPort(int assocID) {

  unsigned short respPort=0;
  unsigned short ua_id=0;

  /* Get the ua_id for this association first */
  if (sctpd_state_assocID_to_ua_id(assocID,&ua_id)==0) {
    /* Get the Resp Port number */
    if ((sctpd_state_get_udp_resp_port(ua_id,&respPort)==0)) 
      return respPort;
  }
    
  return 0;

}


unsigned short getUDPRespPortForInstance(unsigned short instanceID) {

  unsigned short respPort=0;
  unsigned short ua_id=0;

  /* Get the ua_id for this instance first */
  if (sctpd_state_instanceID_to_ua_id(instanceID,&ua_id)==0) {
    /* Get the Resp Port number */
    if ((sctpd_state_get_udp_resp_port(ua_id,&respPort)==0)) 
      return respPort;
  }
    
  return 0;


}

unsigned short getUDPNotifPortForUA(unsigned short UA_ID) {

  unsigned short notifPort=0;

  if (sctpd_state_get_udp_notif_port(UA_ID,&notifPort)==0)
    return notifPort;
  else
    return 0;


}


unsigned short getUDPRespPortForUA(unsigned short UA_ID) {

  unsigned short respPort=0;

  if (sctpd_state_get_udp_resp_port(UA_ID,&respPort)==0)
    return respPort;
  else
    return 0;


}



/* ----------------------------------------------------------------------------*/














