/*
 *
 * SCTP implementation according to RFC 2960.
 * Copyright (C) 2000 by Siemens AG, Munich, Germany.
 *
 * Realized in co-operation between Siemens AG
 * and University of Essen, Institute of Computer Networking Technology.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * There are two mailinglists available at http://www.sctp.de which should be
 * used for any discussion related to this implementation.
 *
 * Contact: discussion@sctp.de
 *          Michael.Tuexen@icn.siemens.de
 *          ajung@exp-math.uni-essen.de
 *
 * Purpose: Module (header) contains functions to handle timer events
 * Author : Sim Woon Chiat Siemens ICM N PG U SE D2
 *
 */



#ifndef TIMER_EVENT_H
#define TIMER_EVENT_H

#include <time.h>

#define HEARTBEAT_TIMER       100
#define HEARTBEAT_INT_TIMER   200


typedef
struct timer_event_struct {

  unsigned int timer_id;
  time_t time_expired;
  unsigned short timer_type;
  unsigned short ua_id;
  unsigned short status;


} TimerEvent;


/* Insert a timer_event into State and call sctp_startTimer func of the 
   SCTP library.
   tEvent : pointer to the event to be inserted
   RETURN : id of timer if success, 0 failure
*/
int insertTimerEvent(TimerEvent *tEvent);

/* Remove a specific timer
   timer_id : ID of timer event to remove
   RETURN : 1 success, 0 failure
*/
int removeTimerEvent(unsigned int timer_id);

/* Find a particular TimerEvent 
   timer_id : ID of timer event to search for
   tEvent : Pointer to data structure found.
   RETURN : 1 success, 0 failure
*/
int findTimerEvent (unsigned int timer_id,TimerEvent **tEvent);

/* Find the first TimerEvent on the list for a particular UA
   us_id : ID of UA to search for
   RETURN : 1 success, 0 failure
*/
int findTimerEventForUA (unsigned ua_id);


/* Stop a specific timer
   timer_id : ID of timer event to stop
   RETURN : 1 success, 0 failure
*/
int stopTimerEvent(unsigned int timer_id);


#endif
