/*
 *
 * SCTP implementation according to RFC 2960.
 * Copyright (C) 2000 by Siemens AG, Munich, Germany.
 *
 * Realized in co-operation between Siemens AG
 * and University of Essen, Institute of Computer Networking Technology.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * There are two mailinglists available at http://www.sctp.de which should be
 * used for any discussion related to this implementation.
 *
 * Contact: discussion@sctp.de
 *          Michael.Tuexen@icn.siemens.de
 *          ajung@exp-math.uni-essen.de
 *
 * Purpose: Provides state keeping for echo_server_monitor
 * Author : Sim Woon Chiat Siemens ICM N PG U SE D2
 *
 */


#include <stdio.h>
#include "sctp_info_utils.h"

// storage for association info: assocID,associationStatus etc

assocInfo assocInfoArr[MAX_ASSOCIATIONS_MONITORED];
unsigned int noAssoc=0;


// FUNCTION DEFINITIONS -----------------------------------------------------

int check_assoc_present(int assocID)
{

  int i;
  for (i=0;i<noAssoc;i++)
  {
    if (assocInfoArr[i].assocID==assocID)
      return i;
  }
  return -1;

}


int insert_assocID(int assocID)
{
  if (noAssoc<MAX_ASSOCIATIONS_MONITORED)
  {

    assocInfoArr[noAssoc++].assocID=assocID;
    assocInfoArr[noAssoc++].no_paths=0;

  }

}


int insert_assocInfo(int assocID,SCTP_AssociationStatus assocStatus)
{

  if (noAssoc<MAX_ASSOCIATIONS_MONITORED)
  {

    assocInfoArr[noAssoc].assocID=assocID;
    assocInfoArr[noAssoc].assocStatus=assocStatus;

  }

}

// Stupid way of doing it... 
int remove_assocInfo(int assocID)
{

  int i=0,k;
  int row;

  if ((i=check_assoc_present(assocID))!=0) 
  {
    row = assocInfoArr[i].row;

    for (k=0;k<noAssoc;k++)
    {
      assocInfoArr[k]=assocInfoArr[k+1];

      // This is a stupid hack to lower each an everyone row number after 
      // the deleted one... so that the remaining have the correct row numbers.
      if (assocInfoArr[k].row>=row)
	assocInfoArr[k].row--;
    }

    //assocInfoArr[noAssoc--]=NULL;

  }

}


int update_assocInfo(int assocID,SCTP_AssociationStatus assocStatus)
{

  int i=0;

  if ((i=check_assoc_present(assocID))!=-1) 
  {
    assocInfoArr[i].assocStatus=assocStatus;
    return 1;
  }

  return 0;
}


assocInfo * get_assocInfo(assocID)
{

  int i=0;

  if ((i=check_assoc_present(assocID))!=-1) 
  {
    
    return &assocInfoArr[i];
  }

  return NULL;

}


int update_assocRow(int assocID,int row)
{
  int i=0;

  if ((i=check_assoc_present(assocID))!=-1) 
  {
    assocInfoArr[i].row=row;
    return 1;
  }

  return 0;


}

int get_assocRow(int assocID)
{
  int i=0;

  if ((i=check_assoc_present(assocID))!=-1) 
  {
    return assocInfoArr[i].row;

  }

  return -1;
}


int insert_pathID(int assocID,int pathID)
{

  assocInfo * associnfo;
  if ((associnfo = get_assocInfo(assocID))!=NULL)
  {
    if (associnfo->no_paths< MAX_PATHS_PER_ASSOC)
    {
      associnfo->paths[associnfo->no_paths++].pathID=pathID;
      return 1;
    }    
  }
  return 0;
}


int check_path_present(int assocID,int pathID)
{
  assocInfo * associnfo;
  int i;

  if ((associnfo = get_assocInfo(assocID))!=NULL)
  {

    for (i=0;i<associnfo->no_paths;i++)
      {
	if (associnfo->paths[i].pathID==pathID)
	  return i;
      }
    return -1;
      
  }

  return -1;
}


int insert_pathInfo(int assocID,int pathID,SCTP_PathStatus pathStatus)
{

  assocInfo * associnfo;
  if ((associnfo = get_assocInfo(assocID))!=NULL)
  {

    if (associnfo->no_paths< MAX_PATHS_PER_ASSOC)
    {

      associnfo->paths[associnfo->no_paths].pathID=pathID;
      associnfo->paths[associnfo->no_paths].pathStatus=pathStatus;
      associnfo->no_paths++;
      return 1;
    }
  
  }

  return 0;

}

int remove_pathInfo(int assocID,int pathID)
{

  assocInfo * associnfo;
  if ((associnfo = get_assocInfo(assocID))!=NULL)
  {

  
  }


}

int update_pathInfo(int assocID,int pathID,SCTP_PathStatus pathStatus)
{

  int i=0;
  assocInfo * associnfo;

  if ((associnfo = get_assocInfo(assocID))!=NULL)
  {

    if ((i=check_path_present(assocID,pathID))!=-1) 
    {
	associnfo->paths[i].pathStatus=pathStatus;
	return 1;
    }
  
  }
  return 0;

}

pathInfo * get_pathInfo(int assocID,int pathID)
{
  int i;
  assocInfo * associnfo;
  if ((associnfo = get_assocInfo(assocID))!=NULL)
  {
    if ((i=check_path_present(assocID,pathID))!=-1) 
    {
    
      return &(associnfo->paths[i]);
    }
      
  }
  return NULL;


}






