/*
 *
 * SCTP implementation according to RFC 2960.
 * Copyright (C) 2000 by Siemens AG, Munich, Germany.
 *
 * Realized in co-operation between Siemens AG
 * and University of Essen, Institute of Computer Networking Technology.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * There are two mailinglists available at http://www.sctp.de which should be
 * used for any discussion related to this implementation.
 *
 * Contact: discussion@sctp.de
 *          Michael.Tuexen@icn.siemens.de
 *          ajung@exp-math.uni-essen.de
 *
 * Purpose: Header for sctp_info-utils.c
 * Author : Sim Woon Chiat Siemens ICM N PG U SE D2
 *
 */


#ifndef SCTPINFOUTILS_H
#define SCTPINFOUTILS_H

#include "sctp.h"

#define MAX_ASSOCIATIONS_MONITORED      20
#define MAX_PATHS_PER_ASSOC             20

// Some data structures used to keep track of the associations
typedef 
struct path_Info {
  int pathID;
  SCTP_PathStatus pathStatus;

}pathInfo;

typedef
struct assoc_Info {
  int assocID;
  SCTP_AssociationStatus assocStatus;
  int no_paths;
  int row;          // the row number of this association in the assocListBox
                    // should not be here actually since this is UI info...
  pathInfo paths[MAX_PATHS_PER_ASSOC];

}assocInfo;

// Functions for accessing and modifying association information
int insert_assocID(int assocID);
int check_assoc_present(int assocID);
int insert_assocInfo(int assocID,SCTP_AssociationStatus assocStatus);
int remove_assocInfo(int assocID);
int update_assocInfo(int assocID,SCTP_AssociationStatus assocStatus);
assocInfo * get_assocInfo(int assocID);
int update_assocRow(int assocID,int row);
int get_assocRow(int assocID);

// Functions for accessing and modifying path information
int insert_pathID(int assocID,int pathID);
int check_path_present(int assocID,int pathID);
int insert_pathInfo(int assocID,int pathID,SCTP_PathStatus pathStatus);
int remove_pathInfo(int assocID,int pathID);
int update_pathInfo(int assocID,int pathID,SCTP_PathStatus pathStatus);
pathInfo * get_pathInfo(int assocID,int pathID);



#endif
