/*
 *
 * SCTP implementation according to RFC 2960.
 * Copyright (C) 2000 by Siemens AG, Munich, Germany.
 *
 * Realized in co-operation between Siemens AG
 * and University of Essen, Institute of Computer Networking Technology.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * There are two mailinglists available at http://www.sctp.de which should be
 * used for any discussion related to this implementation.
 *
 * Contact: discussion@sctp.de
 *          Michael.Tuexen@icn.siemens.de
 *          ajung@exp-math.uni-essen.de
 *
 * Daemon-based terminal program with GUI
 * Author : Sim Woon Chiat Siemens ICM N PG U SE D2
 *
 */


#include <stdio.h>
#include <stdlib.h>         /* for atoi() under Linux */

#include <errno.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <fcntl.h>
#include <signal.h>

#include "../sctpdlib/ua.h"

#include <glib.h>
#include <gtk/gtk.h>

#include "icons.h"

/* DEFINITION OF CONSTANTS ------------------------------------------ */
#define DEFAULT_X_SIZE    500
#define DEFAULT_Y_SIZE    800

#define SCTP_App_Port     0    /* Terminal Program use random port */
#define MAXIMUM_NUMBER_OF_LOCAL_ADDRESSES 5

/* NETWORK VARIABLES ------------------------------------------------ */
static int udpfd;                    // file descriptor to the UDP
static short sctpInstance=0;
static short connected = 0;
static unsigned int curAssocID = 0;
static int UDP_Notif_Port= 6234;
static int UDP_Resp_Port= 6235;

/* GTK VARIABLES ---------------------------------------------------- */
GtkWidget *mainWindow;
GtkWidget *termWindow;
GtkWidget *statusWindow;
GtkWidget *connectDialogWin;
GtkWidget *disconnectDialogWin;
GtkWidget *ipEntry, *portEntry;
GtkWidget *textEntry;

/* Other variables */
static unsigned char localAddressList[MAXIMUM_NUMBER_OF_LOCAL_ADDRESSES][SCTP_MAX_IP_LEN];
static unsigned short noOfLocalAddresses = 0;
static int unknownCommand = 0;
static unsigned char destAddr[255];
static unsigned char destPort[5];

/* EXIT functions  */
void cleanUpAndExit() {

  /* Unregister Instance with sctpd */
   sctpd_unregisterInstance(sctpInstance);

  /* Stop sctpdlib */
  sctpd_stopLibrary();

  gtk_main_quit();

}


void CloseTerminal (GtkWidget * window, gpointer data){

  cleanUpAndExit();

}

/* Arguments Functions -------------------------- */
void printUsage(void) {

  printf("Usage:   sctpd_terminal [options] -s source_addr_1 -s source_addr_2 ...\n");
  printf("Options : \n");
  printf("r     : response port\n");
  printf("a     : app port\n");

}

void getArgs(int argc, char **argv) {

    int c;
    extern char *optarg;
    extern int optind;

    while ((c = getopt(argc, argv, "s:r:a:")) != -1)
    {
        switch (c) {
        case 's':
	  if ((noOfLocalAddresses < MAXIMUM_NUMBER_OF_LOCAL_ADDRESSES) &&
	      (strlen(optarg) < SCTP_MAX_IP_LEN  )) {
	    strcpy(localAddressList[noOfLocalAddresses], optarg);
	    noOfLocalAddresses++;
	  }
	  break;
	case 'a':
	  UDP_Notif_Port = atoi(optarg);
	  break;
	case 'r':
	  UDP_Resp_Port = atoi(optarg);
	  break;
        default:
            unknownCommand = 1;
            break;
        }
    }
}

void checkArgs(void) {
    int abortProgram;
    int printUsageInfo;
    
    abortProgram = 0;
    printUsageInfo = 0;
    
    if (noOfLocalAddresses == 0) {
        printf("Error:   At least one sourceaddress must be specified.\n");
        abortProgram = 1;
        printUsageInfo = 1;
    }
    if (unknownCommand ==1) {
         printf("Error:   Unkown options in command.\n");
         printUsageInfo = 1;
    }
    
    if (printUsageInfo == 1)
        printUsage();
    if (abortProgram == 1)
        exit(-1);
}



/* GUI Functions  ------------------------------- */
void displayStatus(char *status){

  gtk_container_add(GTK_CONTAINER(statusWindow),
		    gtk_list_item_new_with_label(status));
  gtk_widget_show_all(statusWindow);

}

void displayText(char *status) {


  gtk_container_add(GTK_CONTAINER(termWindow),
		    gtk_list_item_new_with_label(status));
  gtk_widget_show_all(termWindow);


}


// Code canabalize from book ------------------------------------------- 
/*
 * CloseDialog
 *
 * Close the dialog window.  The dialog handle is passed
 * in as the data.
 */
void CloseDialog (GtkWidget *widget, gpointer data) {

    /* --- Close it --- */
    gtk_widget_destroy (GTK_WIDGET (data));

}

/*
 * ClosingDialog
 *
 * Calls when window is about to close.  Returns FALSE
 * to let it close.  Releases the grab which made the dialog
 * window modal.
 */
void ClosingDialog (GtkWidget *widget, gpointer data) {

    gtk_grab_remove (GTK_WIDGET (widget));

}



/*
 * Popup
 *
 * Display a popup dialog window with a message and an
 * "Ok" button
 */
void Popup (char *szMessage) {

    static GtkWidget *label;
    GtkWidget *button;
    GtkWidget *dialog_window;

    /* --- Create a dialog window --- */
    dialog_window = gtk_dialog_new ();

    /* --- Trap the window close signal to release the grab --- */
    gtk_signal_connect (GTK_OBJECT (dialog_window), "destroy",
                     GTK_SIGNAL_FUNC (ClosingDialog),
                     &dialog_window);

    /* --- Add a title to the window --- */
    gtk_window_set_title (GTK_WINDOW (dialog_window), "Popup");

    /* --- Create a small border --- */
    gtk_container_border_width (GTK_CONTAINER (dialog_window), 5);

    /* -------------------------- */
    /* --- Create the message --- */
    /* -------------------------- */

    /* --- Create the message in a label --- */
    label = gtk_label_new (szMessage);

    /* --- Put some room around the label --- */
    gtk_misc_set_padding (GTK_MISC (label), 10, 10);

    /* --- Add the label to the dialog box --- */
    gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog_window)->vbox),
                        label, TRUE, TRUE, 0);

    /* --- Make the label visible --- */
    gtk_widget_show (label);

    /* ------------------- */
    /* --- "Ok" button --- */
    /* ------------------- */

    /* --- Create the "Ok" button --- */
    button = gtk_button_new_with_label ("Ok");

    /* --- Need to close the window if they press "Ok" --- */
    gtk_signal_connect (GTK_OBJECT (button), "clicked",
                     GTK_SIGNAL_FUNC (CloseDialog),
                     dialog_window);

    /* --- Allow it to be the default button --- */
    GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT);

    /* --- Add the button to the dialog box --- */
    gtk_box_pack_start (GTK_BOX (
                  GTK_DIALOG (dialog_window)->action_area),
                 button, TRUE, TRUE, 0);

    /* --- Make the button the default button --- */
    gtk_widget_grab_default (button);

    /* --- Make the button visible --- */
    gtk_widget_show (button);

    /* --- Make the dialog box visible --- */
    gtk_widget_show (dialog_window);

    gtk_grab_add (dialog_window);
}

/* -- End of canabalization ---------------------------------------------- */


void connectToPeer (GtkWidget * window, gpointer data) {

  char tstr[255];

  strcpy(destAddr,gtk_entry_get_text(GTK_ENTRY(ipEntry)));
  strcpy(destPort,gtk_entry_get_text(GTK_ENTRY(portEntry)));
  
  sprintf(tstr,"Connecting to peer with IP address = %s at port = %s",
	  destAddr,destPort);
  displayStatus(tstr);
  
  sctpd_associate(sctpInstance, 
		  5, 
		  destAddr, 
		  atoi(destPort), 
		  NULL);

  /* Remove the dialog box */
  gtk_grab_remove (GTK_WIDGET (data));
  gtk_widget_destroy(GTK_WIDGET (data));

}


void disconnectFromPeer (GtkWidget * WINDOW, gpointer data) {

  displayStatus("Disconnecting from peer");

  if (curAssocID!=0) {
    sctpd_shutdown(curAssocID);    
  }

  /* Remove the dialog box */
  gtk_grab_remove (GTK_WIDGET (data));
  gtk_widget_destroy(GTK_WIDGET (data));

}



/*
 * connectDialog
 *
 * Display a connection Dialog
 */
void  connectDialog (GtkMenuItem * menuitem,gpointer data) {

    GtkWidget *label;
    GtkWidget *connectBtn, *cancelBtn;
    GtkWidget *table, *hbox, *vbox;
    
    /* --- Create a dialog window --- */
    connectDialogWin = gtk_dialog_new ();

    /* --- Trap the window close signal to release the grab --- */
    gtk_signal_connect (GTK_OBJECT (connectDialogWin), "destroy",
			GTK_SIGNAL_FUNC (ClosingDialog),
			&connectDialogWin);

    /* --- Add a title to the window --- */
    gtk_window_set_title (GTK_WINDOW (connectDialogWin), "Connect to:");

    /* --- Create a small border --- */
    gtk_container_border_width (GTK_CONTAINER (connectDialogWin), 5);

    /* --- Create the message in a label --- */
    label = gtk_label_new ("Enter the destination information");

    /* --- Put some room around the label --- */
    /* gtk_misc_set_padding (GTK_MISC (label), 10, 10); */

    /* --- Add the label to the dialog box --- */
    gtk_box_pack_start (GTK_BOX (GTK_DIALOG (connectDialogWin)->vbox),
                        label, TRUE, TRUE, 0);

    /* Create the vbox */
    vbox = gtk_vbox_new(FALSE,8);

    /* Packing table to hold text entries and their labels */
    table = gtk_table_new(2,2,FALSE);

    /* IP Address */
    label = gtk_label_new("IP Address");
    gtk_table_attach(GTK_TABLE(table),label,0,1,0,1,GTK_FILL,GTK_FILL,0,0);
    ipEntry = gtk_entry_new();
    gtk_entry_set_text(GTK_ENTRY(ipEntry),destAddr);
    gtk_table_attach(GTK_TABLE(table),ipEntry,1,2,0,1,GTK_FILL,GTK_FILL,0,0);

    /* Port Number */
    label = gtk_label_new("Port");
    gtk_table_attach(GTK_TABLE(table),label,0,1,1,2,GTK_FILL,GTK_FILL,0,0);
    portEntry = gtk_entry_new();
    gtk_entry_set_text(GTK_ENTRY(portEntry),destPort);
    gtk_table_attach(GTK_TABLE(table),portEntry,1,2,1,2,GTK_FILL,GTK_FILL,0,0);

    /* Add the table to the vbox */
    gtk_box_pack_start (GTK_BOX (vbox),
			table, TRUE, TRUE, 0);

    /* --- Create the "Connect" button --- */
    connectBtn = gtk_button_new_with_label ("Connect");

    gtk_signal_connect (GTK_OBJECT (connectBtn), "clicked",
                     GTK_SIGNAL_FUNC (connectToPeer),
                     connectDialogWin);



    /* --- Allow it to be the default button --- */
    //GTK_WIDGET_SET_FLAGS (connectBtn, GTK_CAN_DEFAULT);

    /* --- Make the button the default button --- */
    //gtk_widget_grab_default (connectBtn);


    /* --- Create the "Cancel" button --- */
    cancelBtn = gtk_button_new_with_label ("Cancel");

    /* --- Need to close the window if they press "cancel" --- */
    gtk_signal_connect (GTK_OBJECT (cancelBtn), "clicked",
                     GTK_SIGNAL_FUNC (CloseDialog),
                     connectDialogWin);


    /* --- Add the buttons to the hbox --- */
    hbox = gtk_hbox_new(FALSE,5);
    gtk_box_pack_start (GTK_BOX (hbox),
			connectBtn, TRUE, TRUE, 0);
    gtk_box_pack_start (GTK_BOX (hbox),
			cancelBtn, TRUE, TRUE, 0);

    /* Add the hbox to the vbox */
    gtk_box_pack_start (GTK_BOX (vbox),
			hbox, TRUE, TRUE, 0);

    gtk_box_pack_start (GTK_BOX (GTK_DIALOG (connectDialogWin)->action_area),
			vbox, TRUE, TRUE, 0);
    
    /* --- Make the dialog box visible --- */
    gtk_widget_show_all (connectDialogWin);

    gtk_grab_add (connectDialogWin);
}



/*
 * disconnectDialog
 *
 * Display a disconnection Dialog
 */
void  disconnectDialog (GtkMenuItem * menuitem,gpointer data){

    GtkWidget *label;
    GtkWidget *disconnectBtn, *cancelBtn;
    GtkWidget *hbox;
    
    /* --- Create a dialog window --- */
    disconnectDialogWin = gtk_dialog_new ();

    /* --- Trap the window close signal to release the grab --- */
    gtk_signal_connect (GTK_OBJECT (disconnectDialogWin), "destroy",
			GTK_SIGNAL_FUNC (ClosingDialog),
			&disconnectDialogWin);

    /* --- Add a title to the window --- */
    gtk_window_set_title (GTK_WINDOW (disconnectDialogWin), "Connect to:");

    /* --- Create a small border --- */
    gtk_container_border_width (GTK_CONTAINER (disconnectDialogWin), 5);

    /* --- Create the message in a label --- */
    label = gtk_label_new ("Are you sure you wish to disconnect?");

    /* --- Add the label to the dialog box --- */
    gtk_box_pack_start (GTK_BOX (GTK_DIALOG (disconnectDialogWin)->vbox),
                        label, TRUE, TRUE, 0);


    /* --- Create the "Disconnect" button --- */
    disconnectBtn = gtk_button_new_with_label ("Disconnect");

    gtk_signal_connect (GTK_OBJECT (disconnectBtn), "clicked",
                     GTK_SIGNAL_FUNC (disconnectFromPeer),
                     disconnectDialogWin);


    /* --- Create the "Cancel" button --- */
    cancelBtn = gtk_button_new_with_label ("Cancel");

    /* --- Need to close the window if they press "cancel" --- */
    gtk_signal_connect (GTK_OBJECT (cancelBtn), "clicked",
                     GTK_SIGNAL_FUNC (CloseDialog),
                     disconnectDialogWin);


    /* --- Add the buttons to the hbox --- */
    hbox = gtk_hbox_new(FALSE,5);
    gtk_box_pack_start (GTK_BOX (hbox),
			disconnectBtn, TRUE, TRUE, 0);
    gtk_box_pack_start (GTK_BOX (hbox),
			cancelBtn, TRUE, TRUE, 0);


    gtk_box_pack_start (GTK_BOX (GTK_DIALOG (disconnectDialogWin)->action_area),
			hbox, TRUE, TRUE, 0);
    
    /* --- Make the dialog box visible --- */
    gtk_widget_show_all (disconnectDialogWin);

    gtk_grab_add (disconnectDialogWin);
}



void menuExitHandler(GtkMenuItem * menuitem,gpointer data) {

  
  cleanUpAndExit();

}



/* UDP Callback Handlers --------------------------------------------------------------- */

void* communicationUpNotif(unsigned int assocID, unsigned short status,
                           int noOfPaths,unsigned short noOfInStreams, 
			   unsigned short noOfOutStreams, void* dummy) {

  int i;
  unsigned short noAddr=0;

  displayStatus("Reveived a CommunicationUpNotif from SCTP layer");
  connected = 1;
  curAssocID = assocID;

  return NULL;
}


void dataArriveNotif(unsigned int assocID, unsigned int streamID, 
		     unsigned int len, unsigned int protoID, 
		     unsigned int unordered, void* ulpDataPtr) {
  
  char tstr[255],data[255];
  int length;


#ifdef DEBUG
  printf("DataArrive Notification Received from daemon\n");
#endif

  sprintf(tstr,"Data Arrived from assocID=%d, stream = %d , size = %d",
	  assocID,streamID,len);

  sctpd_receive(assocID,streamID,data,&length,SCTP_MSG_DEFAULT);

  /* Add a null-terminating character here */
  data[length]= (char) 0;

  displayText(data);

}


void shutdownCompleteNotif(unsigned int assocID, void* ulpDataPtr) {

#ifdef DEBUG
  printf("ShutdownComplete Notification Received from daemon\n");
#endif

  displayStatus("Shutdown completed message received!");

  connected=0;
  curAssocID = 0;

}


void networkStatusChangeNotif(unsigned int assocID, short affectedPathID, 
			      unsigned short newState, void* ulpDataPtr){

  displayStatus("Network status changed Notification message received!");
  
  /* Alert user */
  Popup("Network Status Changed");

}


void communicationLostNotif(unsigned int assocID, 
			    unsigned short status, 
			    void* ulpDataPtr) {

  char tstr[255];

  sprintf(tstr,"Communication Lost for association (ID=%d)",assocID);
  displayStatus(tstr);

  connected=0;
  curAssocID = 0;
}



void udp_recv_callback (gpointer data,gint source,GdkInputCondition condition) {

  char recvbuff[256];
  socklen_t len;
  struct sockaddr_in taddr;
  ssize_t mesglen;

  mesglen = recvfrom(source,recvbuff,sizeof(recvbuff),0,(struct sockaddr *) &taddr,&len);

  if (mesglen>0) {
    /* call the dispatcher provided by sctpudputils which will
       call the registered UDP callback functions. */

    dispatchUAMesg(recvbuff);

  }


}



/* connect to the UDP port */
int initializeUDPListening(int port) {

  int sockflags;
  struct sockaddr_in uaddr;
  struct sockaddr_in paddr;

  udpfd = socket(AF_INET, SOCK_DGRAM,0);

  bzero (&uaddr,sizeof(uaddr));
  bzero (&paddr,sizeof(paddr));

  uaddr.sin_family = AF_INET;
  uaddr.sin_port = htons(port);
  uaddr.sin_addr.s_addr=htonl(INADDR_ANY);
  
  if ((bind(udpfd, (struct sockaddr *) &uaddr, sizeof(uaddr)))==0) {

    // set socket as non-blocking
    sockflags = fcntl (udpfd,F_GETFL,0);
    fcntl (udpfd,F_SETFL, sockflags | O_NONBLOCK);

    // register the fd with the event handler of GTK
    gdk_input_add(udpfd,GDK_INPUT_READ,udp_recv_callback,NULL);

    displayStatus("Sucessfully binded to the application UDP port");
    return 1;
  }

  else {

    displayStatus("unable to bind to UDP port");
    return 0;

  }

}


/* Callback when Enter key pressed in Text entry */
void sendText(GtkWidget * widget, GtkWidget * entry) {

  gchar *textToSend=NULL;
  char tstr[1024];
  int result;

  if (connected) {
    /* Extract the text from the Entry widget */
    // bzero((void *)textToSend,strlen(textToSend));
    bzero((void *)tstr,1024);

    textToSend = gtk_entry_get_text(GTK_ENTRY(entry));

    /* Call the daemon to send */

    result = sctpd_send(curAssocID,
			1,
			textToSend, strlen(textToSend),
			SCTP_GENERIC_PAYLOAD_PROTOCOL_ID,
			SCTP_USE_PRIMARY, SCTP_NO_CONTEXT, SCTP_INFINITE_LIFETIME, 
			SCTP_ORDERED_DELIVERY, SCTP_BUNDLING_DISABLED);
    

    /* Display in status window */
    sprintf(tstr,"Sending : %s",textToSend);
    displayStatus(tstr);

    /* Clear the entry */
    gtk_entry_set_text(GTK_ENTRY(entry),"");

  }
  else {
    /* Display Error message in status window */

    displayStatus("There is no connection");

  }
    
}



void BuildInterface() {

  GtkWidget *hbox1;
  GtkWidget *hbox2;
  GtkWidget *menubar;
  GtkWidget *actionMenu;
  GtkWidget *actionMenuItem;
  GtkWidget *menuitem;
  GtkWidget *termScrollWin;  
  GtkWidget *statusScrollWin;  
  GtkWidget *hpaned;  
  GtkWidget *toolbar;  
  
  /* create and set the parameters of the window */
  mainWindow = gtk_window_new (GTK_WINDOW_TOPLEVEL);
  gtk_window_set_position(GTK_WINDOW(mainWindow), GTK_WIN_POS_CENTER);
  gtk_window_set_default_size (GTK_WINDOW(mainWindow), DEFAULT_X_SIZE,DEFAULT_Y_SIZE);
  gtk_window_set_title (GTK_WINDOW(mainWindow), "SCTP Terminal");

  /* connect the signal handlers for the window */
  gtk_signal_connect (GTK_OBJECT(mainWindow), "destroy", GTK_SIGNAL_FUNC(CloseTerminal), NULL);

  /* create the vertical packing boxes */
  hbox1 = gtk_vbox_new (FALSE,2);
  hbox2 = gtk_vbox_new (FALSE,5);

  gtk_container_add (GTK_CONTAINER(mainWindow), hbox1);

  /* Menu bar ------------------------------------------------------------*/
  menubar=gtk_menu_bar_new();
  gtk_box_pack_start (GTK_BOX(hbox1), menubar, FALSE, FALSE, 0);
 
  /* Create Action Menu item on menu bar */
  actionMenuItem = gtk_menu_item_new_with_label("Action");
  gtk_menu_bar_append(GTK_MENU_BAR(menubar),actionMenuItem);
  

  /* Action Menu ----------------------------------*/
  actionMenu = gtk_menu_new();
  gtk_menu_item_set_submenu(GTK_MENU_ITEM(actionMenuItem),actionMenu);

  /* Action Menu Item */
  menuitem =  gtk_menu_item_new_with_label("Connect");
  gtk_menu_append(GTK_MENU(actionMenu),menuitem);
  gtk_signal_connect(GTK_OBJECT(menuitem),"activate",GTK_SIGNAL_FUNC(connectDialog),NULL);  

  /* Action Menu Item */
  menuitem =  gtk_menu_item_new_with_label("Exit");
  gtk_menu_append(GTK_MENU(actionMenu),menuitem);
  gtk_signal_connect(GTK_OBJECT(menuitem),"activate",GTK_SIGNAL_FUNC(menuExitHandler),NULL);  


  /* Toolbar ---------------------------------------------------------------------------------*/
  toolbar = gtk_toolbar_new(GTK_ORIENTATION_HORIZONTAL,GTK_TOOLBAR_BOTH);
  gtk_box_pack_start (GTK_BOX(hbox1), toolbar, FALSE, TRUE, 0);
  gtk_toolbar_append_space(GTK_TOOLBAR(toolbar));

  /* Tool Item */
  gtk_toolbar_append_item ( GTK_TOOLBAR(toolbar), NULL, "Connect",NULL,
			    CreateWidgetFromXpm(mainWindow,(gchar **) connect_Xpm),
			    (GtkSignalFunc) connectDialog,NULL);
  gtk_toolbar_append_item ( GTK_TOOLBAR(toolbar), NULL, "Disconnect",NULL,
			    CreateWidgetFromXpm(mainWindow,(gchar **) disconnect_Xpm),
			    (GtkSignalFunc) disconnectDialog,NULL);

  /* Pane */
  hpaned = gtk_vpaned_new();
  gtk_paned_add1(GTK_PANED(hpaned),hbox2);
  gtk_box_pack_start (GTK_BOX(hbox1), hpaned, TRUE, TRUE, 0);

  /* Terminal Window */
  
  termWindow = gtk_list_new();   
  termScrollWin = gtk_scrolled_window_new(GTK_ADJUSTMENT(gtk_adjustment_new (0,0,0,0,0,0)),
					  GTK_ADJUSTMENT(gtk_adjustment_new (0,0,0,0,0,0)));
  gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW(termScrollWin),termWindow);
  gtk_box_pack_start (GTK_BOX(hbox2), termScrollWin, TRUE, TRUE, 0);
 

  /* Text entry */
  textEntry = gtk_entry_new();
  gtk_box_pack_start (GTK_BOX(hbox2), textEntry, FALSE, FALSE, 0);

  gtk_signal_connect(GTK_OBJECT(textEntry), "activate",
		     GTK_SIGNAL_FUNC(sendText), textEntry);

  /* Status Window */
  
  statusWindow = gtk_list_new();   
  statusScrollWin = gtk_scrolled_window_new(GTK_ADJUSTMENT(gtk_adjustment_new (0,0,0,0,0,0)),
					       GTK_ADJUSTMENT(gtk_adjustment_new (0,0,0,0,0,0)));
  gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW(statusScrollWin),statusWindow);
  gtk_paned_add2(GTK_PANED(hpaned),statusScrollWin);
 
  gtk_paned_set_position(GTK_PANED(hpaned),500);

  /* display the window on the screen */
  gtk_widget_show_all(mainWindow);  

}



gint main (gint argc, gchar * argv[]) {

  UDP_UA_CB udp_callbacks;
  char tstr[255];

  /* Trap Ctrl-C */
  signal(SIGINT, cleanUpAndExit);

  /* handle all command line options */
  getArgs(argc, argv);
  checkArgs();

  /* Initialize Dest addr and port */
  strcpy(destAddr,"");
  strcpy(destPort,"");

  /* Required GTK initialization call */
  gtk_init (&argc, &argv);

  /* Build the user interface */
  BuildInterface();

  /* Register the UDP callbacks with the dispatch mechanism */
  udp_callbacks.communicationUpNotif = &communicationUpNotif;
  udp_callbacks.shutdownCompleteNotif = &shutdownCompleteNotif;
  udp_callbacks.networkStatusChangeNotif = &networkStatusChangeNotif;
  udp_callbacks.dataArriveNotif = &dataArriveNotif;

  /* Start listening on the UDP port on which the daemon sends notification */
  initializeUDPListening(UDP_Notif_Port);

  /* Initialize sctpdlib */
  if (sctpd_initLibrary(UDP_Notif_Port,UDP_Resp_Port)==1) {

    /* Call sctpd_registerInstance */
    sctpInstance = sctpd_registerInstance(SCTP_App_Port,10,10,noOfLocalAddresses,
					  localAddressList,udp_callbacks);

    sprintf(tstr,"Instance ID = %d",sctpInstance);
    displayStatus(tstr);

    if (sctpInstance!=0) {
      /* Start the GTK loop */
      gtk_main();
    }
    else {
      printf("Unable to register an instance with sctpd\n");
      exit(1);
    }
  }
  else {
    printf("Unable to initialize sctpdlib\n");
    exit(1);
  }

  return 0;

}






