/*
 *  $Id: sctpudputils.c,v 1.5 2001/11/13 18:29:38 ajung Exp $
 *
 * SCTP implementation according to RFC 2960.
 * Copyright (C) 2000 by Siemens AG, Munich, Germany.
 *
 * Realized in co-operation between Siemens AG
 * and University of Essen, Institute of Computer Networking Technology.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * There are two mailinglists available at http://www.sctp.de which should be
 * used for any discussion related to this implementation.
 *
 * Contact: discussion@sctp.de
 *          Michael.Tuexen@icn.siemens.de
 *          ajung@exp-math.uni-essen.de
 *
 * Purpose: UDP utility functions for the SCTP daemon program
 * Author : Sim Woon Chiat ICM N PG U SE D2
 */

#include <stdio.h>

#include <errno.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <stdlib.h>

#include "sctp.h"
#include "sctpudputils.h"
#include "ua.h"

#define MAXIMUM_NUMBER_OF_LOCAL_ADDRESSES    10

UDP_SCTPD_CB *sctpd_udpCB;
UDP_UA_CB *ua_udpCB;

UDP_SCTPD_CB udp_sctpd_cb;
UDP_UA_CB udp_ua_cb;

static unsigned char *localhostAddr          = "127.0.0.1";


/*
int register_UA_UDP_Callbacks(UDP_UA_CB * udpcb) {

  fprintf(stdout,"Registering UDP callbacks for UA\n");
  fflush(stdout);

  ua_udpCB = udpcb;

  return 1;
}
*/


int register_UA_UDP_Callbacks(UDP_UA_CB  udpcb) {

#ifdef DEBUG
  fprintf(stdout,"Registering UDP callbacks for UA\n");
  fflush(stdout);
#endif

  udp_ua_cb = udpcb;

  return 1;
}

int register_UA_UDP_Callbacks2(UDP_UA_CB udpcb) {

  udp_ua_cb = udpcb;

  return 1;
}

/*
int register_SCTPD_UDP_Callbacks(UDP_SCTPD_CB * udpcb) {

  fprintf(stdout,"Registering UDP callbacks for daemon\n");
  fflush(stdout);

  sctpd_udpCB = udpcb;

  return 1;
}

*/


int register_SCTPD_UDP_Callbacks(UDP_SCTPD_CB udpcb) {

#ifdef DEBUG
  fprintf(stdout,"Registering UDP callbacks for daemon\n");
  fflush(stdout);
#endif

  udp_sctpd_cb = udpcb;

  return 1;
}


int sendnRawData(unsigned int port,unsigned char mesg[MAX_UDP_DATA_LENGTH],int length) {

  struct sockaddr_in puiaddr;     
  static int fdServer;

  if (port>0) {

    if ((fdServer = socket(AF_INET, SOCK_DGRAM,0))!=-1) {
      bzero (&puiaddr,sizeof(puiaddr));
      puiaddr.sin_family = AF_INET;
      puiaddr.sin_port = htons(port);
      if (inet_aton(localhostAddr, &puiaddr.sin_addr)) {
	if (sendto(fdServer,mesg,length,0,(struct sockaddr *) &puiaddr,
		   sizeof(puiaddr))!=-1) {
	  close(fdServer);
	  return 1;
	}
	else {
	  fprintf(stdout,"Failure in sending message to peer\n");
	  fprintf(stdout,"Error = %s\n",strerror(errno));
	  fflush(stdout);
	  close(fdServer);	
	}       
      }
      else {
	close(fdServer);
      }
    }
    else {
      fprintf(stdout,"Error in creating socket\n");
      fprintf(stdout,"Error = %s\n",strerror(errno));
      fflush(stdout);      
    }
  }

  return 0;
}


int create_func_call(fnCall *funcCall,unsigned short func_id) {

  static int t_call_id=0;

  /* generate random number for the call_id
     for now, just keep adding */

  t_call_id++;

  funcCall->fs.func_id = func_id;   /* Should check if ID is valid? */
  funcCall->fs.call_id = t_call_id;

  /* initialize the no. of arguements to zero first...*/
  funcCall->fs.no_of_args=0;

  return t_call_id;
}


/* utility function to add a arguement to an existing fnCall structure */
int add_func_arg(fnCall *funcCall,unsigned short arg_type,
		 unsigned short arg_size, void *arg_ptr) {
  
  if (funcCall!=NULL) {

    /* initialize the argSpec structure for this arguement */
    (funcCall->as[funcCall->fs.no_of_args]).arg_type=arg_type;
    (funcCall->as[funcCall->fs.no_of_args]).arg_size=arg_size;

    /* copy the pointer of the arguement into dataptr array */
    funcCall->dataptr[funcCall->fs.no_of_args]=arg_ptr;

  }

  funcCall->fs.no_of_args++;

  return 1;

}



/* read the individual element of the structure and pack the 
   info in sequence into the char buffer */
int pack_function_call(fnCall *funcCall, char buf[MAX_UDP_DATA_LENGTH]) {

  int p,no_args,i;
  size_t fsize,size_of_arg;
  unsigned short arg_type;

  argSpec tas;

  /* initialize p to point to the beginning of the buff */
  p = 0;
  fsize = 0;

  /* clear the buffer to 0 */
  bzero (buf,MAX_UDP_DATA_LENGTH);

  /* Take note of the number of arguements */
  no_args = (funcCall->fs).no_of_args;

  /* Change the byte order of the fields */
  funcCall->fs.call_id = htons(funcCall->fs.call_id);
  funcCall->fs.func_id = htons(funcCall->fs.func_id);
  funcCall->fs.no_of_args = htons(funcCall->fs.no_of_args);

  /* copy the call_id field into the buffer */
  fsize = sizeof(funcCall->fs.call_id);
  memcpy(&buf[p],&(funcCall->fs).call_id,fsize);
  p=p+fsize;

  /* copy the func_id field into the buffer */
  fsize = sizeof(funcCall->fs.func_id);
  memcpy(&buf[p],&(funcCall->fs).func_id,fsize);
  p=p+fsize;

  /* copy the no of args fields into the buffer */ 
  fsize = sizeof(funcCall->fs.no_of_args);
  memcpy(&buf[p],&(funcCall->fs).no_of_args,fsize);
  p = p + fsize;


  /* loop through the arguements specs and copy each argsSpec struct into buffer
   as well as the arguements */
  for (i=0;i<no_args;i++) {
    
    tas = funcCall->as[i];
    
    /* Note the size and type of the arguement first */
    size_of_arg = tas.arg_size;   
    arg_type = tas.arg_type;

    /* Convert the byte order of the fields */
    tas.arg_type = htons(tas.arg_type);
    tas.arg_size = htons(tas.arg_size);  

    /* Copy the arguementSpec into the buffer */
    fsize=sizeof(tas);

    memcpy (&buf[p],&tas,fsize);
    p = p + fsize;

    /* Check the type of the arg, and if integer, change the 
       byte order b4 storing into buffer */
    switch (arg_type) {
    case ARG_TYPE_SHORT: case ARG_TYPE_INT: case ARG_TYPE_LONG:
      
      /* Compare the size to the system type's size. */
      if (size_of_arg==sizeof(short))
	*((short *)funcCall->dataptr[i]) = htons(*((short *)funcCall->dataptr[i]));
      else if(size_of_arg==sizeof(long))
	*((long *)funcCall->dataptr[i]) = htonl(*((long *)funcCall->dataptr[i]));
      break;
    }

    /* Copy the arguement into the buffer */
    memcpy (&buf[p],funcCall->dataptr[i],size_of_arg);
    p = p + size_of_arg;

    /* Need to convert the data back to the correct format as they
       might still be needed by the app */       
    switch (arg_type) {
    case ARG_TYPE_SHORT: case ARG_TYPE_INT: case ARG_TYPE_LONG:
      
      /* Compare the size to the system type's size. */
      if (size_of_arg==sizeof(short))
	*((short *)funcCall->dataptr[i]) = ntohs(*((short *)funcCall->dataptr[i]));
      else if(size_of_arg==sizeof(long))
	*((long *)funcCall->dataptr[i]) = ntohl(*((long *)funcCall->dataptr[i]));
      break;
    }
  }
    
  return p+1;

}


int unpack_function_call(char buf[MAX_UDP_DATA_LENGTH],fnCall *funcCall) {

  int p,no_args,i;
  size_t fsize,size_of_arg;
  unsigned short arg_type;

  argSpec tas;
  void *argptr ;


  /* initialize p to point to the beginning of the buff */
  p = 0;
  fsize = 0;

  /* copy the call_id field into the buffer */
  fsize = sizeof(funcCall->fs.call_id);
  memcpy(&(funcCall->fs).call_id,&buf[p],fsize);
  p=p+fsize;

  /* copy the func_id field into the buffer */
  fsize = sizeof(funcCall->fs.func_id);
  memcpy(&(funcCall->fs).func_id,&buf[p],fsize);
  p=p+fsize;

  /* copy the no of args fields into the buffer */
  fsize = sizeof(funcCall->fs.no_of_args);
  memcpy(&(funcCall->fs).no_of_args,&buf[p],fsize);
  p = p + fsize;

  /* Change the byte order of the fields */
  funcCall->fs.call_id = ntohs(funcCall->fs.call_id);
  funcCall->fs.func_id = ntohs(funcCall->fs.func_id);
  funcCall->fs.no_of_args = ntohs(funcCall->fs.no_of_args);
  
  /* Take note of the number of args */
  no_args = (funcCall->fs).no_of_args;

  /* loop through the arguements and copy each argsSpec struct into buffer
     allocate memory for the argument values. */
  for (i=0;i<no_args;i++) {
    
    fsize=sizeof(tas);
    memcpy (&tas,&buf[p],fsize);

    /* Convert the byte order of the fields  */
    tas.arg_type = ntohs(tas.arg_type);
    tas.arg_size = ntohs(tas.arg_size); 

    /* Note down the size and type of the arg */
    size_of_arg = tas.arg_size;
    arg_type = tas.arg_type;

    funcCall->as[i] = tas;
    p = p + fsize;    

    /* allocate the amount of memory required to store the data */
    argptr = (void *)malloc(size_of_arg);

    if (argptr!=NULL) {
      /* copy the data to the allocated memory... 
	 maybe should do a check here to see the memory has been sucessfully allocated. */
      memcpy(argptr,&buf[p],size_of_arg);
      p = p + size_of_arg;

      /* store the pointer to the data into funcCall struct */
      funcCall->dataptr[i]=argptr;

      /* Check the type of the arg, and if integer, change the 
	 byte order b4 storing into buffer */
      switch (arg_type) {
      case ARG_TYPE_SHORT: case ARG_TYPE_INT: case ARG_TYPE_LONG:
      
	/* Compare the size to the system type's size. */
	if (size_of_arg==sizeof(short))
	  *((short *)funcCall->dataptr[i]) = ntohs(*((short *)funcCall->dataptr[i]));
	else if(size_of_arg==sizeof(long))
	  *((long *)funcCall->dataptr[i]) = ntohl(*((long *)funcCall->dataptr[i]));
	
	break;
      }

      /* store the pointer in the internal list... for future admin purposes. */
      //ptr_list[ptr_list_size++]=argptr;

    }
  }

  return 1;

}



int free_func_args(fnCall *funcCall) {

  int i;

  for (i=0;i<funcCall->fs.no_of_args;i++) {
    if (funcCall->dataptr[i])
      free(funcCall->dataptr[i]);
  }

  return 1;
}


int dispatchSCTPDMesg(unsigned char mesg[MAX_UDP_DATA_LENGTH]) {

  fnCall fc;
  /* Special variables to be used for sctpd_registerInstance */
  unsigned char localAddressList[MAXIMUM_NUMBER_OF_LOCAL_ADDRESSES][SCTP_MAX_IP_LEN];
  unsigned int noLocalAddr=0;
  int i=0;

  /* Parse the UDP message into the function data structure */
  unpack_function_call(mesg,&fc);

  /* Look at the func_id to determine the response to take */
  switch (fc.fs.func_id) {

  case FN_SCTPD_INITLIBRARY:
    if (udp_sctpd_cb.handle_sctpd_initLibrary!=NULL)
      udp_sctpd_cb.handle_sctpd_initLibrary(*((unsigned short *)fc.dataptr[0]),
					    *((int *)fc.dataptr[1]),
					    *((int *)fc.dataptr[2]));
    break;

  case FN_SCTPD_STOPLIBRARY:
    if (udp_sctpd_cb.handle_sctpd_stopLibrary!=NULL)
      udp_sctpd_cb.handle_sctpd_stopLibrary(*((unsigned short *)fc.dataptr[0]));
    break;

  case FN_SCTP_REGISTER_INSTANCE:

    if (udp_sctpd_cb.handle_sctp_registerInstance!=NULL) {

      /* Regenerate the IP address list */
      /* Get the number of addresses */
      noLocalAddr=*((unsigned int *)fc.dataptr[4]);
      /* check if greater than number of allowable addr */
      if (noLocalAddr>MAXIMUM_NUMBER_OF_LOCAL_ADDRESSES)
	noLocalAddr = MAXIMUM_NUMBER_OF_LOCAL_ADDRESSES;

      /* Build the address list */
      for (i=0;i<noLocalAddr;i++) {
	/* Note: Addresses start on the 6th element */
	// strcpy(localAddressList[i],(unsigned char *)fc.dataptr[i+5]);
	memcpy(localAddressList[i],fc.dataptr[i+5],SCTP_MAX_IP_LEN);

      }

      udp_sctpd_cb.handle_sctp_registerInstance(*((int *)fc.dataptr[0]),    /* The UA ID */
						*((unsigned short *)fc.dataptr[1]),
						*((unsigned short *)fc.dataptr[2]),
						*((unsigned short *)fc.dataptr[3]),
						*((unsigned int *)fc.dataptr[4]),
						localAddressList);
    }

    break;

  case FN_SCTP_UNREGISTER_INSTANCE:
    if (udp_sctpd_cb.handle_sctp_unregisterInstance!=NULL) 
      udp_sctpd_cb.handle_sctp_unregisterInstance(*((unsigned short *)fc.dataptr[0]));

      break;

  case FN_GET_LOCAL_ADDR:
    if (udp_sctpd_cb.handle_get_local_addr!=NULL)
      udp_sctpd_cb.handle_get_local_addr(*((unsigned short *)fc.dataptr[0]));

    break;

  case FN_SCTP_ASSOCIATE:
    if (udp_sctpd_cb.handle_sctp_associate!=NULL) 
      udp_sctpd_cb.handle_sctp_associate(*((unsigned short *)fc.dataptr[0]),
					 *((unsigned short *)fc.dataptr[1]),
					 ((unsigned char *)fc.dataptr[2]),
					 *((unsigned short *)fc.dataptr[3]),
					 (void *)fc.dataptr[4]);    
    break;

  case FN_SCTP_SHUTDOWN:
    if (udp_sctpd_cb.handle_sctp_shutdown!=NULL)
      udp_sctpd_cb.handle_sctp_shutdown(*((int *)fc.dataptr[0]));
    break;

  case FN_SCTP_ABORT:
    if (udp_sctpd_cb.handle_sctp_abort!=NULL)
      udp_sctpd_cb.handle_sctp_abort(*((int *)fc.dataptr[0]));
      break;
      
  case FN_SCTP_SEND:
    if (udp_sctpd_cb.handle_sctp_send!=NULL)
      udp_sctpd_cb.handle_sctp_send(*((int *)fc.dataptr[0]),
				    *((short *)fc.dataptr[1]),
				    ((char *)fc.dataptr[2]),
				    *((int *)fc.dataptr[3]),
				    *((int *)fc.dataptr[4]),
				    *((short *)fc.dataptr[5]),
				    (void *)fc.dataptr[6],
				    *((int *)fc.dataptr[7]),
				    *((int *)fc.dataptr[8]),
				    *((int *)fc.dataptr[9]));	                           
    break;

  case FN_SCTP_RECEIVE:
    if (udp_sctpd_cb.handle_sctp_receive!=NULL)
      udp_sctpd_cb.handle_sctp_receive(*((int *)fc.dataptr[0]),
				       *((short *)fc.dataptr[1]),
				       NULL,     /* Ignore the buffer for now */
				       0,    /* TO DO: Maybe should convert to pass a pointer */
				       *((int *)fc.dataptr[2]));
    break;

  case FN_SCTP_GET_ASSOC_STATUS:

    if (udp_sctpd_cb.handle_sctp_getAssocStatus!=NULL)
      udp_sctpd_cb.handle_sctp_getAssocStatus(*((int *)fc.dataptr[0]));
    break;

  case FN_SCTP_GET_ASSOC_DEFAULTS:
    
    if (udp_sctpd_cb.handle_sctp_getAssocDefaults!=NULL)
      udp_sctpd_cb.handle_sctp_getAssocDefaults(*((int *)fc.dataptr[0]));
    break;


  case FN_SCTP_GET_PATH_STATUS:     
    if (udp_sctpd_cb.handle_sctp_getPathStatus!=NULL)
      udp_sctpd_cb.handle_sctp_getPathStatus(*((int *)fc.dataptr[0]),
					     *((short *)fc.dataptr[1]));
    break;


  case FN_SCTP_SET_ASSOC_STATUS    :
    if (udp_sctpd_cb.handle_sctp_setAssocStatus!=NULL)
      udp_sctpd_cb.handle_sctp_setAssocStatus(*((int *)fc.dataptr[0]),
					      ((SCTP_AssociationStatus *)fc.dataptr[1]));

    break;

  case FN_SCTP_SET_PATH_STATUS     :
    if (udp_sctpd_cb.handle_sctp_setPathStatus!=NULL)
      udp_sctpd_cb.handle_sctp_setPathStatus(*((int *)fc.dataptr[0]),
					     *((short *)fc.dataptr[1]),
					     (SCTP_PathStatus *)fc.dataptr[2]);
    break;

  case FN_SCTP_SET_ASSOC_DEFAULTS  :
    if (udp_sctpd_cb.handle_sctp_setAssocDefaults!=NULL)
      udp_sctpd_cb.handle_sctp_setAssocDefaults(*((unsigned short *)fc.dataptr[0]),
						((SCTP_InstanceParameters *)fc.dataptr[1]));
    break;

  case FN_SCTP_GET_PRIMARY         :
    if (udp_sctpd_cb.handle_sctp_getPrimary!=NULL)
      udp_sctpd_cb.handle_sctp_getPrimary(*((unsigned int *)fc.dataptr[0]));
    break;

  case FN_SCTP_SET_PRIMARY         :
    if (udp_sctpd_cb.handle_sctp_setPrimary!=NULL)
      udp_sctpd_cb.handle_sctp_setPrimary(*((int *)fc.dataptr[0]),
					  *((short *)fc.dataptr[1]));
    break;

  case FN_SCTP_GET_SRTT_REPORT     :
    if (udp_sctpd_cb.handle_sctp_getSrttReport!=NULL)
      udp_sctpd_cb.handle_sctp_getSrttReport(*((int *)fc.dataptr[0]),
					     *((short *)fc.dataptr[1]));
    break;

  case FN_SCTP_CHANGE_HEARTBEAT    :
    if (udp_sctpd_cb.handle_sctp_changeHeartBeat!=NULL)
      udp_sctpd_cb.handle_sctp_changeHeartBeat(*((unsigned int *)fc.dataptr[0]),
					       *((short *)fc.dataptr[1]),
					       *((int *)fc.dataptr[2]),
					       *((unsigned int *)fc.dataptr[3]));
    break;

  case FN_SCTP_REQUEST_HEARTBEAT   :
    if (udp_sctpd_cb.handle_sctp_requestHeartbeat!=NULL)
      udp_sctpd_cb.handle_sctp_requestHeartbeat(*((unsigned int *)fc.dataptr[0]),
						*((short *)fc.dataptr[1]));
    break;

  case FN_SCTP_SET_FAILURE_THRES   :
    if (udp_sctpd_cb.handle_sctp_setFailureThreshold!=NULL)
      udp_sctpd_cb.handle_sctp_setFailureThreshold(*((unsigned int *)fc.dataptr[0]),
						   *((short *)fc.dataptr[1]));
    break;

  case FN_SCTP_DELETE_ASSOC        :
    if (udp_sctpd_cb.handle_sctp_deleteAssociation!=NULL)
      udp_sctpd_cb.handle_sctp_deleteAssociation(*((unsigned int *)fc.dataptr[0]));
    break;

  case FN_SCTP_GET_INSTANCE_ID          :
    if (udp_sctpd_cb.handle_sctp_getInstanceID!=NULL)
      (udp_sctpd_cb.handle_sctp_getInstanceID(*((unsigned int *) fc.dataptr[0]),
					      NULL));
    break;

  case FN_SCTP_RECEIVE_UNSENT           :
    if (udp_sctpd_cb.handle_sctp_receiveUnsent!=NULL)
      udp_sctpd_cb.handle_sctp_receiveUnsent(*(unsigned int*)fc.dataptr[0],
					     NULL, /* pointer to char buffer */
					     (unsigned int*)fc.dataptr[1],  
					     (unsigned short*)fc.dataptr[2],
					     (unsigned short*)fc.dataptr[3],
					     (unsigned int*)fc.dataptr[4]);


    break;

  case FN_SCTP_RECEIVE_UNACK            :
    if (udp_sctpd_cb.handle_sctp_receiveUnacked!=NULL)
      udp_sctpd_cb.handle_sctp_receiveUnacked(*(unsigned int*)fc.dataptr[0],
					      NULL, /* pointer to char buffer */
					      (unsigned int*)fc.dataptr[1], 
					      (unsigned short*)fc.dataptr[2],
					      (unsigned short*)fc.dataptr[3],
					      (unsigned int*)fc.dataptr[4]);


    break;

  case FN_SCTP_ADD_IP_ADDRESS           :
    if (udp_sctpd_cb.handle_sctp_addIPAddress!=NULL)
      udp_sctpd_cb.handle_sctp_addIPAddress(*(unsigned int*)fc.dataptr[0],
					    ((char *)fc.dataptr[1]), 
					    (unsigned int*)fc.dataptr[2]);
    break;

  case FN_SCTP_DELETE_IP_ADDRESS        :
    if (udp_sctpd_cb.handle_sctp_deleteIPAddress!=NULL)
      udp_sctpd_cb.handle_sctp_deleteIPAddress(*(unsigned int*)fc.dataptr[0],
					       ((char *)fc.dataptr[1]), 
					       (unsigned int*)fc.dataptr[2]);

    break;

  case FN_SCTP_SET_REMOTE_PRIMARY       :
    if (udp_sctpd_cb.handle_sctp_setRemotePrimary!=NULL)
      udp_sctpd_cb.handle_sctp_setRemotePrimary(*(unsigned int*)fc.dataptr[0],
						((char *)fc.dataptr[1]));

    break;

  case FN_SCTP_GET_NO_LOCAL_ADDR        :
    if (udp_sctpd_cb.handle_sctp_getNumberOfLocalAddresses!=NULL)
      udp_sctpd_cb.handle_sctp_getNumberOfLocalAddresses(*(unsigned int*)fc.dataptr[0]);

  break;

  case FN_SCTP_GET_LOCAL_ADDR           :
    if (udp_sctpd_cb.handle_sctp_getLocalAddress!=NULL)
      udp_sctpd_cb.handle_sctp_getLocalAddress(*(unsigned int*)fc.dataptr[0],
					       *(int *)fc.dataptr[1]);

    break;

  case UDP_HEARTBEAT_ACK               :
    if (udp_sctpd_cb.handle_heartbeat_ack!=NULL)
      udp_sctpd_cb.handle_heartbeat_ack(*((unsigned int *)fc.dataptr[0]));
    break;

  case HEARTBEAT_EXPIRED               :
    if (udp_sctpd_cb.handle_heartbeat_expired!=NULL)
      udp_sctpd_cb.handle_heartbeat_expired(*((unsigned int *)fc.dataptr[0]));
    break;

  case HEARTBEAT_INT_EXPIRED               :
    if (udp_sctpd_cb.handle_heartbeat_int_expired!=NULL)
      udp_sctpd_cb.handle_heartbeat_int_expired(*((unsigned int *)fc.dataptr[0]));
    break;

  default:
    fprintf(stdout,"Unknown function ID %d\n",fc.fs.func_id);
    fflush(stdout);
    free_func_args(&fc);
    return 0;

  }

 
  /* IMPORTANT :
     free memory memory allcated by sctpudputils for the storing of the arguement values */
  free_func_args(&fc);

  return 1;
}


int dispatchUAMesg(unsigned char mesg[MAX_UDP_DATA_LENGTH]) {

  fnCall fc;

  unpack_function_call(mesg,&fc);

  /* Look at the func_id to determine the response to take */
  switch (fc.fs.func_id) {

    case CB_COMMUNICATION_UP_NOTIF:

      if (udp_ua_cb.communicationUpNotif!=NULL)
	udp_ua_cb.communicationUpNotif(*((int *)fc.dataptr[0]),
				      *((unsigned short *)fc.dataptr[1]),
				      *((int *)fc.dataptr[2]),
				      *((unsigned short *)fc.dataptr[3]),
				      *((unsigned short *)fc.dataptr[4]),
				      (void *)fc.dataptr[5]);
      break;

    case CB_NETWORK_STATUS_CHANGE_NOTIF:

      if (udp_ua_cb.networkStatusChangeNotif!=NULL)
	udp_ua_cb.networkStatusChangeNotif(*((int *)fc.dataptr[0]),
				      *((unsigned short *)fc.dataptr[1]),     
				      *((unsigned short *)fc.dataptr[2]),
				      (void *)fc.dataptr[3]);

      break;

    case CB_SHUTDOWN_COMPLETE_NOTIF:
      if (udp_ua_cb.shutdownCompleteNotif!=NULL)
	udp_ua_cb.shutdownCompleteNotif((*((int *)fc.dataptr[0])),
					(void *)fc.dataptr[1]);

      break;

    case CB_DATA_ARRIVE_NOTIF:
      if (udp_ua_cb.dataArriveNotif!=NULL)
	udp_ua_cb.dataArriveNotif(*((int *)fc.dataptr[0]),
				   *((int *)fc.dataptr[1]),
	                           *((int *)fc.dataptr[2]),
			           *((int *)fc.dataptr[3]),
                                   *((int *)fc.dataptr[4]),                                   
	                           (void *)fc.dataptr[5]);


      break;

    case CB_SEND_FAILURE_NOTIF :
      if (udp_ua_cb.sendFailureNotif!=NULL)
	udp_ua_cb.sendFailureNotif(*((int *)fc.dataptr[0]),
				    (char *)fc.dataptr[1],
				    *((int *)fc.dataptr[2]),
				    (int *)fc.dataptr[3],    /* Pointer to context from sctp_send */
				    (void *)fc.dataptr[4]);  /* pointer to upper layer data */
      break;

    case CB_COMMUNICATION_LOST_NOTIF :
      if (udp_ua_cb.communicationLostNotif!=NULL)
	udp_ua_cb.communicationLostNotif(*((int *)fc.dataptr[0]),            /* Association ID */
					  *((unsigned short *)fc.dataptr[1]), /* status */
					  (void *)fc.dataptr[2]);

      break;

    case CB_COMMUNICATION_ERROR_NOTIF :
      if (udp_ua_cb.communicationErrorNotif!=NULL)
	udp_ua_cb.communicationErrorNotif(*((int *)fc.dataptr[0]), /* Association ID */
					  *((unsigned short *)fc.dataptr[1]),
					  (void *)fc.dataptr[2]);  /* pointer to upper layer data */
      break;

    case CB_RESTART_NOTIF :
      if (udp_ua_cb.restartNotif!=NULL)
	udp_ua_cb.restartNotif(*((int *)fc.dataptr[0]), /* Association ID */ 	
			       (void *)fc.dataptr[1]);  /* pointer to upper layer data */
      break;
      
    case CB_QUEUE_STATUS_CHANGE_NOTIF :
      if (udp_ua_cb.queueStatusChangeNotif!=NULL)
	udp_ua_cb.queueStatusChangeNotif(*((unsigned int *)fc.dataptr[0]),      
					 *((int *)fc.dataptr[1]),
					 *((int *)fc.dataptr[2]),
					 *((int *)fc.dataptr[3]),
					 (void *)fc.dataptr[4]);  /* pointer to upper layer data */
      break;

    case CB_ASCONF_STATUS_NOTIF :
      if (udp_ua_cb.asconfStatusNotif!=NULL)
	udp_ua_cb.asconfStatusNotif(*((unsigned int *)fc.dataptr[0]),      
				    *((int *)fc.dataptr[1]),
				    *((int *)fc.dataptr[2]),
				    (void *)fc.dataptr[3], /* pointer to a temp, request specific struct*/
				    (void *)fc.dataptr[4]);/* pointer to upper layer data */
      break;

    case UDP_HEARTBEAT :
      if (udp_ua_cb.handle_heartbeat!=NULL)
	udp_ua_cb.handle_heartbeat(*((unsigned int *)fc.dataptr[0]));
      break;

  }

 
  // IMPORTANT
  // free memory memory allcated by sctpudputils for the storing of the arguement values
  free_func_args(&fc);

  return 1;
}





